/////////////////////////////////////////////////////////////////////////////
//
// @descr    See MainDlg.h
//
// @short    See MainDlg.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External

//2 From SystemMgr
#include <nlClientInterface.h>
#include <nlSystemMgrType.h>
#include <nlCommand.h>

//2 From Utiliites
#include <nlCommonType.h>
#include <nlCompositeHandle.h>

//1 Local
#include "MainDlg.h"

#include "Main.h"
#include "MainDlg.h"
#include "nlEndMachineDlg.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  const int nNumNetworks = 4;
  const int nNumEndMachinesPerNetwork = 3;
  const int nNumEndMachines= nNumNetworks * nNumEndMachinesPerNetwork;
  const int nPofMessageListVisibleLines = 9;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CMainDlg::SnlImp* CMainDlg::Imp()       { return m_pImp; }
inline const  CMainDlg::SnlImp* CMainDlg::Imp() const { return m_pImp; }

struct CMainDlg::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CMainDlg* const pOwner,
    CnlClientInterface* const pClientInterface );

  // Destructor
   ~SnlImp();

  // OK or Cancel
  void Close();

  // Clear all of the machine headers and hide them
  void ResetAllPofs();

  // Disable the points of focus
  void DisableAllPofs();

  // Handle Point of Focus changes
  void HandlePofChange();

  // Add a sequence number to a machine header
  void AppendPofSequenceNum(
    CListBox* const pListPof,
    const CString& strPofNum );
  
  // Enable/Disable the run-control buttons
  void EnableRunControl( 
    bool bEnable );

  // Display the End Machine Dialog corresponding to the selected machine
  void DisplayEndMachineDialog(
    const unsigned int nMachineId );

  // Display the Router Dialog corresponding to the selected machine
  void DisplayRouterDialog();

  // Set the layer filter level
  void SetLayerFilter(
    const int nLayer );

  // Update all model-based displays
  void UpdateAll();

  // Update the layer filter display
  void UpdateSettings();

  // Request command processing advance by a single step
  void SingleStep();

  // Find the specified step in a machine header
  CListBox* FindStep(
    unsigned int nStep ) const;
  
  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CMainDlg* const m_pOwner;

  // Handle to the model
  CnlClientInterface* m_pClientInterface;

  // Handle to the networks
  std::vector< CnlCompositeHandle > m_hNetworks;
  
  // Handles to the end machines, organized by network
  std::vector< CnlCompositeHandle > m_hEndMachines;

  // Handle to the one and only router
  CnlCompositeHandle m_hRouter;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The IP addresses of the networks
  std::vector< CString > m_NetworkIpAddresses;

  // The IP addresses of the end machines
  std::vector< CString > m_MachineIpAddresses;

  // The IP addresses used by the router
  std::vector< CString > m_RouterIpAddresses;

  // The Point of Focus list boxes
  std::vector< unsigned long > m_PofListBoxResourceIds;
  
private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CMainDlg::SnlImp::SnlImp(
  CMainDlg* const pOwner,
  CnlClientInterface* const pClientInterface )
 :
  m_pOwner( pOwner ),
  m_pClientInterface( pClientInterface )
{
  m_NetworkIpAddresses.push_back( _T( "123.234.1.0/24" ) );  // Network I
  m_NetworkIpAddresses.push_back( _T( "123.234.2.0/24" ) );  // Network II
  m_NetworkIpAddresses.push_back( _T( "123.234.3.0/24" ) );  // Network III
  m_NetworkIpAddresses.push_back( _T( "123.234.4.0/24" ) );  // Network IV
  m_MachineIpAddresses.push_back( _T( "123.234.1.1/24" ) );  // Machine A
  m_MachineIpAddresses.push_back( _T( "123.234.1.2/24" ) );  // Machine B
  m_MachineIpAddresses.push_back( _T( "123.234.1.3/24" ) );  // Machine C
  m_MachineIpAddresses.push_back( _T( "123.234.2.1/24" ) );  // Machine D
  m_MachineIpAddresses.push_back( _T( "123.234.2.2/24" ) );  // Machine E
  m_MachineIpAddresses.push_back( _T( "123.234.2.3/24" ) );  // Machine F
  m_MachineIpAddresses.push_back( _T( "123.234.3.1/24" ) );  // Machine G
  m_MachineIpAddresses.push_back( _T( "123.234.3.2/24" ) );  // Machine H
  m_MachineIpAddresses.push_back( _T( "123.234.3.3/24" ) );  // Machine I
  m_MachineIpAddresses.push_back( _T( "123.234.4.1/24" ) );  // Machine J
  m_MachineIpAddresses.push_back( _T( "123.234.4.2/24" ) );  // Machine K
  m_MachineIpAddresses.push_back( _T( "123.234.4.3/24" ) );  // Machine L
  m_RouterIpAddresses.push_back( _T( "123.234.1.254/24" ) );  // Router
  m_RouterIpAddresses.push_back( _T( "123.234.2.254/24" ) );  // Router
  m_RouterIpAddresses.push_back( _T( "123.234.3.254/24" ) );  // Router
  m_RouterIpAddresses.push_back( _T( "123.234.4.254/24" ) );  // Router

  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_A );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_B );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_C );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_D );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_E );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_F );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_G );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_H );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_I );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_J );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_K );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_EM_L );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_LINK_1 );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_LINK_2 );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_LINK_3 );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_LINK_4 );
  m_PofListBoxResourceIds.push_back( IDC_LIST_POF_ROUTER );
}

///////////////////////////////////////////////////////////////////////////////
CMainDlg::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::Close()
{
  m_pClientInterface->DetachObserver( m_pOwner );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::ResetAllPofs()
{
  CListBox* pListPof;

  for ( unsigned int nListBox = 0; nListBox < m_PofListBoxResourceIds.size(); ++ nListBox )
  {
    pListPof = static_cast< CListBox* >( 
      m_pOwner->GetDlgItem( m_PofListBoxResourceIds[ nListBox ]) );
    pListPof->ResetContent();
    pListPof->ShowWindow( SW_HIDE );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::DisableAllPofs()
{
  CListBox* pListPof;

  for ( unsigned int nListBox = 0; nListBox < m_PofListBoxResourceIds.size(); ++ nListBox )
  {
    pListPof = static_cast< CListBox* >( 
      m_pOwner->GetDlgItem( m_PofListBoxResourceIds[ nListBox ] ) );
    pListPof->SetCurSel( -1 );
    pListPof->EnableWindow( false );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::AppendPofSequenceNum(
  CListBox* const pListPof,
  const CString& strPofNum )
{
  if ( pListPof->GetCount() == 0 )
  {
    //3 This is the first entry
    pListPof->AddString( strPofNum );
  }
  else
  {
    //3 Append to existing entries
    CString strSequenceNumbers;
    pListPof->GetText( 0, strSequenceNumbers );
    strSequenceNumbers = strSequenceNumbers + _T(",") + strPofNum;
    pListPof->ResetContent();
    pListPof->AddString( strSequenceNumbers );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::HandlePofChange()
{
  //1 Get the current activity sequence #
  const CnlCompositeHandle hPof = 
    m_pClientInterface->GetPofHandle();

  const unsigned int nPof = 
    m_pClientInterface->GetPofSequenceNum();

  CString strPofNum;

  if ( nPof == 0 )
  {
    //3 The model has indicated the command is finished
    return;
  }
  else
  {
    strPofNum.Format( _T( "%d" ), nPof );
  }

  //1 Get the associated message and display it
  CListBox* pList = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_MESSAGE ) );
  CString& strMessage = m_pClientInterface->GetPofMessage();
  pList->AddString( strMessage );

  int nLineCount = pList->GetCount();
  if (nLineCount > nPofMessageListVisibleLines)
  {
    const int nResult = pList->SetTopIndex(nLineCount - nPofMessageListVisibleLines);
    
    if ( nResult == LB_ERR )
    {
      // Something went wrong
      assert( 0 );
    }
  }
  
  //1 Add the sequence number to the active machine and highlight it
  DisableAllPofs();

  CListBox* pListPof = NULL;

  if ( hPof == m_hEndMachines[ 0 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_A ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 1 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_B ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 2 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_C ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 3 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_D ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 4 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_E ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 5 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_F ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 6 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_G ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 7 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_H ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 8 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_I ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 9 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_J ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 10 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_K ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hEndMachines[ 11 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_EM_L ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hRouter )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_ROUTER ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hNetworks[ 0 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_LINK_1 ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hNetworks[ 1 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_LINK_2 ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hNetworks[ 2 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_LINK_3 ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
  else if ( hPof == m_hNetworks[ 3 ] )
  {
    pListPof = static_cast< CListBox* >( m_pOwner->GetDlgItem( IDC_LIST_POF_LINK_4 ) );
    AppendPofSequenceNum( pListPof, strPofNum );
    pListPof->ShowWindow( SW_SHOW );
    pListPof->EnableWindow( true );
  }
}


///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::EnableRunControl( 
  bool bEnable )
{
  CButton* pHotButton;
  pHotButton = static_cast< CButton* >( m_pOwner->GetDlgItem( ID_BUTTON_STEP ) );
  pHotButton->EnableWindow( bEnable );
  pHotButton = static_cast< CButton* >( m_pOwner->GetDlgItem( ID_BUTTON_FINISH ) );
  pHotButton->EnableWindow( bEnable );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::DisplayEndMachineDialog(
  const unsigned int nMachineId )
{
  CnlEndMachineDialog dlg( 
    m_pClientInterface, 
    m_hEndMachines[ nMachineId ], 
    m_pOwner );
  dlg.DoModal();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::DisplayRouterDialog()
{
  CnlEndMachineDialog dlg( 
    m_pClientInterface, 
    m_hRouter, 
    m_pOwner );
  dlg.DoModal();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::SetLayerFilter(
  const int nLayer )
{
  CComboBox* pLayerFilterCombo = 
    static_cast< CComboBox* >( m_pOwner->GetDlgItem( IDC_COMBO_LAYER_FILTER ) );
  NnlSystemMgrType::EnlLayer eLayer = 
    static_cast< NnlSystemMgrType::EnlLayer >( nLayer + 1 );
  m_pClientInterface->SetLayerFilter( eLayer );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::UpdateAll()
{
  UpdateSettings();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::UpdateSettings()
{
  //2 Update the event settings
  //3 Show send
  const bool bShowSend = 
    m_pClientInterface->GetShowEventType( NnlSystemMgrType::EVENT_TYPE_SEND );
  CButton* pShowSend = 
    static_cast< CButton* >( m_pOwner->GetDlgItem( IDC_CHECK_EVENT_SEND) );
  pShowSend->SetCheck( bShowSend );

  //3 Show receive/accept
  const bool bShowReceiveAccept = 
    m_pClientInterface->GetShowEventType( NnlSystemMgrType::EVENT_TYPE_RECEIVE_ACCEPT );
  CButton* pShowReceiveAccept = 
    static_cast< CButton* >( m_pOwner->GetDlgItem( IDC_CHECK_EVENT_RECEIVE_ACCEPT ) );
  pShowReceiveAccept->SetCheck( bShowReceiveAccept );

  //2 Update the current layer filter
  NnlSystemMgrType::EnlLayer eLayer = m_pClientInterface->GetLayerFilter();
  CComboBox* pLayerFilterCombo = 
    static_cast< CComboBox* >( m_pOwner->GetDlgItem( IDC_COMBO_LAYER_FILTER ) );
  pLayerFilterCombo->SetCurSel( eLayer - 1 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::SnlImp::SingleStep()
{
  const NnlSystemMgrType::EnlResult eResult = 
    m_pClientInterface->SingleStep();  
}


///////////////////////////////////////////////////////////////////////////////
CListBox* CMainDlg::SnlImp::FindStep(
  unsigned int nStep ) const
{
  //1 Iterate through the POF lists looking for the specified number
  CListBox* pListBox = NULL;
  bool bFoundIt = false;

  for ( unsigned int nListBox = 0; 
        nListBox < m_PofListBoxResourceIds.size(), ! bFoundIt;
        ++nListBox )
  {
    pListBox = static_cast< CListBox* >( 
      m_pOwner->GetDlgItem( m_PofListBoxResourceIds[ nListBox ] ) );

    if ( pListBox->GetCount() > 0 )
    {
      //1 Search this string for the desire step
      CString strSequenceNumbers;
      pListBox->GetText( 0, strSequenceNumbers );
      LPTSTR pstrWhole = strSequenceNumbers.GetBuffer( 0 );
      TCHAR pcSeparator[] = _T( "," );

      //2 Parse the address and length strings
      CString strStep = _tcstok( pstrWhole, pcSeparator );

      while ( ( strStep != NnlType::STR_NULL ) && ! bFoundIt )
      {
        if ( static_cast< unsigned int >( atoi( strStep ) ) == nStep )
        {
          //3 Found it
          bFoundIt = true;
        }

        strStep =_tcstok( NULL, pcSeparator );
      }
       
      strSequenceNumbers.ReleaseBuffer( -1 );  
    }
  }

  return pListBox;
}
/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
  public:
    CAboutDlg();

    // Dialog Data
    //{{AFX_DATA(CAboutDlg)
    enum { IDD = IDD_ABOUTBOX };
    //}}AFX_DATA

    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CAboutDlg)

  protected:

    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

    // Implementation

    //{{AFX_MSG(CAboutDlg)
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
  //{{AFX_DATA_INIT(CAboutDlg)
  //}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CAboutDlg)
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
  //{{AFX_MSG_MAP(CAboutDlg)
    // No message handlers
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////

CMainDlg::CMainDlg(
  CnlClientInterface* const pClientInterface,
  CWnd* pParent  /*=NULL*/ )
 : CDialog(CMainDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CMainDlg)
  //}}AFX_DATA_INIT

  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

  m_pImp = new SnlImp( this, pClientInterface );
}

///////////////////////////////////////////////////////////////////////////////
CMainDlg::~CMainDlg()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CMainDlg)
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CMainDlg, CDialog)
  //{{AFX_MSG_MAP(CMainDlg)
  ON_WM_SYSCOMMAND()
  ON_WM_DESTROY()
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_A, OnButtonEndMachineA)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_B, OnButtonEndMachineB)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_C, OnButtonEndMachineC)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_D, OnButtonEndMachineD)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_E, OnButtonEndMachineE)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_F, OnButtonEndMachineF)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_G, OnButtonEndMachineG)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_H, OnButtonEndMachineH)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_I, OnButtonEndMachineI)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_J, OnButtonEndMachineJ)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_K, OnButtonEndMachineK)
  ON_BN_CLICKED(IDC_BUTTON_END_MACHINE_L, OnButtonEndMachineL)
  ON_BN_CLICKED(IDC_BUTTON_ROUTER, OnButtonRouter)
  ON_BN_CLICKED(ID_BUTTON_STEP, OnButtonStep)
  ON_BN_CLICKED(ID_BUTTON_FINISH, OnButtonFinish)
  ON_BN_CLICKED(ID_BUTTON_TEST, OnButtonTest)
  ON_CBN_SELCHANGE(IDC_COMBO_LAYER_FILTER, OnSelchangeComboLayerFilter)
  ON_BN_CLICKED(IDC_CHECK_EVENT_SEND, OnCheckEventSend)
  ON_BN_CLICKED(IDC_CHECK_EVENT_RECEIVE_ACCEPT, OnCheckEventReceiveAccept)
  ON_LBN_SELCHANGE(IDC_LIST_POF_MESSAGE, OnSelchangeListPofMessage)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CMainDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  Imp()->m_pClientInterface->AttachObserver( this );

  // Add "About..." menu item to system menu.

  // IDM_ABOUTBOX must be in the system command range.
  ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
  ASSERT(IDM_ABOUTBOX < 0xF000);

  CMenu* pSysMenu = GetSystemMenu(FALSE);

  if (pSysMenu != NULL)
  {
    CString strAboutMenu;
    strAboutMenu.LoadString(IDS_ABOUTBOX);

    if (!strAboutMenu.IsEmpty())
    {
      pSysMenu->AppendMenu(MF_SEPARATOR);
      pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
    }
  }

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);  // Set big icon
  SetIcon(m_hIcon, FALSE);  // Set small icon

  //1 Load up the layer filter combo box
  CComboBox* pLayerFilterCombo = 
    static_cast< CComboBox* >( GetDlgItem( IDC_COMBO_LAYER_FILTER ) );

  for ( int nLayer = NnlSystemMgrType::LAYER_UNKNOWN + 1;
       nLayer < NnlSystemMgrType::NUM_LAYERS_PLUS_1;
       ++nLayer )
  {
    NnlSystemMgrType::EnlLayer eLayer = 
      static_cast< NnlSystemMgrType::EnlLayer >( nLayer );
    const CString& strLayer = Imp()->m_pClientInterface->GetLayerString( eLayer );
    pLayerFilterCombo->AddString( strLayer);
  }        

  Imp()->EnableRunControl( false );

  //1 Create the internet
  NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->MakeRouter(
    Imp()->m_hRouter );
  assert( eResult == NnlSystemMgrType::RESULT_OK );

  for ( unsigned int nNetwork = 0; nNetwork < nNumNetworks; ++nNetwork )
  {
    //2 Create a network
    //3 Create
    CnlCompositeHandle hNetwork;
    eResult = Imp()->m_pClientInterface->AddNetworkToInternet( 
        hNetwork,
        Imp()->m_NetworkIpAddresses[ nNetwork ] );
    assert( eResult == NnlSystemMgrType::RESULT_OK );

    //3 Save handle to this network
    Imp()->m_hNetworks.push_back( hNetwork );

    //2 Add a NIC to the router and connect it to this network 
    //3 Create
    eResult = Imp()->m_pClientInterface->AddNicToRouter(
      Imp()->m_hRouter,
      NnlSystemMgrType::ROUTER_NIC_TYPE_NETWORK );
    assert( eResult == NnlSystemMgrType::RESULT_OK );

    //3 Connect
    eResult = Imp()->m_pClientInterface->ConnectRouter( 
      hNetwork,
      nNetwork );  // NIC ID
    assert( eResult == NnlSystemMgrType::RESULT_OK );

    //3 Assign IP
    eResult = Imp()->m_pClientInterface->AssignIpAddress(
      Imp()->m_hRouter,
      nNetwork,  // NIC ID
      Imp()->m_RouterIpAddresses[ nNetwork ] );
    assert( eResult == NnlSystemMgrType::RESULT_OK );

    //2 Create the end machines for this network
    for ( unsigned int nEndMachine = nNetwork * nNumEndMachinesPerNetwork; 
          nEndMachine < ( nNetwork + 1 )* nNumEndMachinesPerNetwork; 
          ++nEndMachine )
    {
      //3 Create
      CnlCompositeHandle hEndMachine;
      eResult = Imp()->m_pClientInterface->AddProcessingMachineToNetwork( 
        hEndMachine,
        hNetwork );
      assert( eResult == NnlSystemMgrType::RESULT_OK );

      //3 Save the handle to this machine
      Imp()->m_hEndMachines.push_back( hEndMachine );
      
      //3 Assign IP address to this end machines
      eResult = Imp()->m_pClientInterface->AssignIpAddress( 
        Imp()->m_hEndMachines[ nEndMachine ],
        0,  // All end machines have only 1 NIC
        Imp()->m_MachineIpAddresses[ nEndMachine ] );
      assert( eResult == NnlSystemMgrType::RESULT_OK );

      //3 Note: They automatically connect their NIC
    }
  }

  Imp()->UpdateAll();
  Imp()->m_pClientInterface->EnablePofProcessing( true );

  return TRUE;  // return TRUE unless you set the focus to a control
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnOK() 
{
  Imp()->Close();
  CDialog::OnOK();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnCancel() 
{
  Imp()->Close();
  CDialog::OnCancel();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::Update(
  CnlModel* const pModel,
  const int eChangeType )
{
  NnlSystemMgrType::EnlMessage eMessage = 
    static_cast< NnlSystemMgrType::EnlMessage >( eChangeType );

  switch( eMessage )
  {
    case NnlSystemMgrType::MESSAGE_MODEL_CHANGED :
    {
      Imp()->UpdateAll();
      break;
    }
    case NnlSystemMgrType::MESSAGE_SETTINGS_CHANGED :
    {
      Imp()->UpdateSettings();
      break;
    }
    case NnlSystemMgrType::MESSAGE_COMMAND_STARTED :
    {
      Imp()->ResetAllPofs();
      CListBox* pListMessage = 
        static_cast< CListBox* >( GetDlgItem( IDC_LIST_POF_MESSAGE ) );
      pListMessage->ResetContent();
      Imp()->EnableRunControl( true );
      break;
    }
    case NnlSystemMgrType::MESSAGE_COMMAND_COMPLETED :
    {
      Imp()->EnableRunControl( false );
      //2 Select the last message
      CListBox* pListMessage = 
        static_cast< CListBox* >( GetDlgItem( IDC_LIST_POF_MESSAGE ) );
      const int nSize = pListMessage->GetCount();
      pListMessage->SetCurSel( nSize -1 );
      OnSelchangeListPofMessage();
      break;
    }
    case NnlSystemMgrType::MESSAGE_POF_CHANGED :
    {
      Imp()->HandlePofChange();
      break;
    }
    default : break;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::StopObserving(
  CnlModel* const pModel )
{
  if ( pModel == reinterpret_cast< CnlModel* >( Imp()->m_pClientInterface ) )
  {
    Imp()->m_pClientInterface = NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
  if ((nID & 0xFFF0) == IDM_ABOUTBOX)
  {
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  }
  else
  {
    CDialog::OnSysCommand(nID, lParam);
  }
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnDestroy()
{
  WinHelp(0L, HELP_QUIT);
  CDialog::OnDestroy();
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnPaint() 
{
  if (IsIconic())
  {
    CPaintDC dc(this); // device context for painting

  SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

    // Center icon in client rectangle
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;

    // Draw the icon
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
  {
    CDialog::OnPaint();
  }
}

///////////////////////////////////////////////////////////////////////////////
// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CMainDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineA() 
{
  Imp()->DisplayEndMachineDialog( 0 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineB() 
{
  Imp()->DisplayEndMachineDialog( 1 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineC() 
{
  Imp()->DisplayEndMachineDialog( 2 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineD() 
{
  Imp()->DisplayEndMachineDialog( 3 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineE() 
{
  Imp()->DisplayEndMachineDialog( 4 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineF() 
{
  Imp()->DisplayEndMachineDialog( 5 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineG() 
{
  Imp()->DisplayEndMachineDialog( 6 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineH() 
{
  Imp()->DisplayEndMachineDialog( 7 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineI() 
{
  Imp()->DisplayEndMachineDialog( 8 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineJ() 
{
  Imp()->DisplayEndMachineDialog( 9 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineK() 
{
  Imp()->DisplayEndMachineDialog( 10 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonEndMachineL() 
{
  Imp()->DisplayEndMachineDialog( 11 );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonRouter() 
{
  Imp()->DisplayRouterDialog();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonStep() 
{
  Imp()->SingleStep();
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonFinish() 
{
  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->RunToEnd();  
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnButtonTest() 
{
  Imp()->m_pClientInterface->Test();  
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnSelchangeComboLayerFilter() 
{
  CComboBox* pLayerFilterCombo = 
    static_cast< CComboBox* >( GetDlgItem( IDC_COMBO_LAYER_FILTER ) );
  Imp()->SetLayerFilter( pLayerFilterCombo->GetCurSel() );
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnCheckEventSend() 
{
  CButton* pShowSend = 
    static_cast< CButton* >( GetDlgItem( IDC_CHECK_EVENT_SEND ) );
  const bool bShowSend =
    pShowSend->GetCheck() != 0;

  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->SetShowEventType(
      NnlSystemMgrType::EVENT_TYPE_SEND,
      bShowSend );  
}

///////////////////////////////////////////////////////////////////////////////
void CMainDlg::OnCheckEventReceiveAccept() 
{
  CButton* pShowReceiveAccept = 
    static_cast< CButton* >( GetDlgItem( IDC_CHECK_EVENT_RECEIVE_ACCEPT ) );
  const bool bShowReceiveAccept =
    pShowReceiveAccept->GetCheck() != 0;

  const NnlSystemMgrType::EnlResult eResult = 
    Imp()->m_pClientInterface->SetShowEventType(
      NnlSystemMgrType::EVENT_TYPE_RECEIVE_ACCEPT,
      bShowReceiveAccept );  
}

void CMainDlg::OnSelchangeListPofMessage() 
{
  CListBox* pMessageList = 
    static_cast< CListBox* >( GetDlgItem( IDC_LIST_POF_MESSAGE ) );
  const unsigned int nStep = static_cast< unsigned int >( pMessageList->GetCurSel() );
  CListBox* pListBox = Imp()->FindStep( nStep + 1 );
  assert( pListBox!= NULL );
  Imp()->DisableAllPofs();
  pListBox->EnableWindow( true );
  pListBox->SetCurSel( 0 );
}
