//////////////////////////////////////////////////////////////////////////////
//
// @descr    Main control panel for end machines.
//
// @short    Main control panel for end machines.
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_END_MACHINE_DIALOG__H__
#define __NL_END_MACHINE_DIALOG__H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From Utilities
#include "nlObserver.h"

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlCompositeHandle;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlEndMachineDialog : public CDialog, public CnlObserver
{
  public:
    //1 Final

    // @descr   Constructor
    // @param   hEndMachine  IN: Handle to the end machine to control
    // @end
    explicit CnlEndMachineDialog(
      CnlClientInterface* const pClientInterface,
      CnlCompositeHandle hEndMachine,
      CWnd* pParent = NULL );

    // @descr   Destructor
    // @end
    virtual ~CnlEndMachineDialog();

    //2 From CnlObserver
    void Update(
      CnlModel* const,
      const int );
    void StopObserving(
      CnlModel* const );
      
    // Dialog Data
    //{{AFX_DATA(CnlEndMachineDialog)
    enum { IDD = IDD_DIALOG_END_MACHINE };
    // NOTE: the ClassWizard will add data members here
    //}}AFX_DATA

  protected:

    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CnlEndMachineDialog)
    virtual BOOL OnInitDialog();
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

    // Generated message map functions
    //{{AFX_MSG(CnlEndMachineDialog)
    virtual void OnOK();
    virtual void OnCancel();
    afx_msg void OnButtonSendArp();
    afx_msg void OnButtonArpA();
    afx_msg void OnButtonArpD();
    afx_msg void OnButtonSendPing();
    afx_msg void OnButtonNetstatRn();
    afx_msg void OnSelchangeComboPort();
    afx_msg void OnButtonPower();
    afx_msg void OnButtonAssignIp();
    //}}AFX_MSG
    
    DECLARE_MESSAGE_MAP()

    //{{AFX_INSERT_LOCATION}}
    // Microsoft Visual C++ will insert additional declarations immediately before the previous line.

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlEndMachineDialog( const CnlEndMachineDialog& );
    CnlEndMachineDialog& operator=( const CnlEndMachineDialog& );
    CnlEndMachineDialog* operator&();
    const CnlEndMachineDialog* operator&() const;
};

#endif
