/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlIntegerPool.h
//
// @short    See nlIntegerPool.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlIntegerPool.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//using namespace xx;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlIntegerPool::SnlImp* CnlIntegerPool::Imp()       { return m_pImp; }
inline const  CnlIntegerPool::SnlImp* CnlIntegerPool::Imp() const { return m_pImp; }

struct CnlIntegerPool::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlIntegerPool* const pOwner,
    const unsigned long nMaxPoolSize );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlIntegerPool* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The maximum number of values that can be in use at one time.
  const unsigned long m_nMaxPoolSize;

  // The pool
  std::set< unsigned long > m_Pool;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlIntegerPool::SnlImp::SnlImp(
  CnlIntegerPool* const pOwner,
  const unsigned long nMaxPoolSize )
 :
  m_pOwner( pOwner ),
  m_nMaxPoolSize( nMaxPoolSize )    
{
}

///////////////////////////////////////////////////////////////////////////////
CnlIntegerPool::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlIntegerPool::CnlIntegerPool(
  const unsigned long nMaxPoolSize )
{
  m_pImp = new SnlImp( this, nMaxPoolSize );
}

///////////////////////////////////////////////////////////////////////////////
CnlIntegerPool::~CnlIntegerPool()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlIntegerPool::IsAvailable() const
{
  return ( Imp()->m_Pool.size() < Imp()->m_nMaxPoolSize );
}

///////////////////////////////////////////////////////////////////////////////
unsigned long CnlIntegerPool::CheckOut()
{
  //1 Error checking
  if ( ! IsAvailable() )
  {
    //2 No values available!
    return 0;
  }

  //1 Get the lowest available value  
  std::set< unsigned long >::const_iterator iter;
  unsigned long nValue = 0;

  for ( nValue = 1; nValue < Imp()->m_nMaxPoolSize; ++nValue )
  {
    iter = Imp()->m_Pool.find( nValue );
    
    if ( iter == Imp()->m_Pool.end() )
    {
      // This value is available
      Imp()->m_Pool.insert( nValue );
      return nValue;
    }
  }

  //1 Could not find any available values
  //3 This should never happen
  assert( 0 );
  return 0;
}

///////////////////////////////////////////////////////////////////////////////
void CnlIntegerPool::CheckIn(
  unsigned long nValue )
{
  Imp()->m_Pool.erase( nValue );
}


