///////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlMfcThread.h
//
// @short    See nlMfcThread.h
//
// @author   Jim Carole
//
// @end
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
///////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External

//1 Internal
#include "nlWinThread.h"
#include "nlMfcThread.h"
#include "nlThreadHandler.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
//
// Insulated Data
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlMfcThread::SnlImp* CnlMfcThread::Imp()       { return m_pImp; }
inline const  CnlMfcThread::SnlImp* CnlMfcThread::Imp() const { return m_pImp; }

struct CnlMfcThread::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlMfcThread* const pOwner );

  // Destructor
   ~SnlImp();

  // Is this a valid handle?
  bool IsValidHandle( 
    HANDLE hHandle );

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlMfcThread* const mOwner;
  
  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')
  std::auto_ptr<CnlWinThread> pWinThread;
  HANDLE  hHandle;
  __int32 nStatus;
  std::auto_ptr<CnlThreadHandler> pThreadHandler;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlMfcThread::SnlImp::SnlImp(
  CnlMfcThread* const pOwner )
 :
  mOwner( pOwner )
{
  hHandle = 0;
  nStatus = ERROR_INVALID_HANDLE;
}

///////////////////////////////////////////////////////////////////////////////
CnlMfcThread::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
bool CnlMfcThread::SnlImp::IsValidHandle( HANDLE hHandle )
{
  return ((hHandle != NULL) && (hHandle != INVALID_HANDLE_VALUE));
}

///////////////////////////////////////////////////////////////////////////////
//
// Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlMfcThread::CnlMfcThread(
  CnlThreadHandler* pThreadHandler,
  unsigned __int32 nInitFlnls,
  LPSECURITY_ATTRIBUTES pSecurityAttributes,
  unsigned __int32 nStackSize )
{
  m_pImp = new SnlImp( this );
  
  Imp()->pWinThread.reset( new CnlWinThread( this ) );

  // detect errors and throw exceptions...
  if (Imp()->pWinThread.get() != 0)
  {
   SetStatus( NO_ERROR );
  }
  else
  {
    SetStatus( ::GetLastError() );
  }

  Imp()->pThreadHandler.reset( pThreadHandler );

  if (GetWinThread()->CreateThread( nInitFlnls, nStackSize, pSecurityAttributes ) == 0)
  {
    SetStatus( ::GetLastError() );
  }

  SetHandle( GetWinThread()->m_hThread );
}

///////////////////////////////////////////////////////////////////////////////
CnlMfcThread::~CnlMfcThread()
{
  // This is necessary because CnlWinThread closes the thread handle
  // and to avoid problems when CnlKernel tries to close the same handle,
  // this call indicates to CnlKernel not to attempt to close the handle.
  SetHandle( 0 );

  if ( Imp()->IsValidHandle( GetHandle() ) == true)
  {
    ::CloseHandle( GetHandle() );
  }
  delete Imp();
}

///////////////////////////////////////////////////////////////////////////////
HANDLE CnlMfcThread::GetHandle() const
{
  return Imp()->hHandle;
}

///////////////////////////////////////////////////////////////////////////////
void CnlMfcThread::SetStatus( __int32 nStatus )
{
  Imp()->nStatus = nStatus;
}

///////////////////////////////////////////////////////////////////////////////
void CnlMfcThread::SetHandle( HANDLE hHandle )
{
  Imp()->hHandle = hHandle;
}

///////////////////////////////////////////////////////////////////////////////
BOOL CnlMfcThread::InitInstance()
{
  ASSERT(Imp()->pWinThread.get());

  return Imp()->pWinThread->MfcInitInstance();
}

///////////////////////////////////////////////////////////////////////////////
int CnlMfcThread::ExitInstance()
{
  ASSERT(Imp()->pWinThread.get());

  return Imp()->pWinThread->MfcExitInstance();
}

///////////////////////////////////////////////////////////////////////////////
CnlWinThread* CnlMfcThread::GetWinThread() const
{
  return Imp()->pWinThread.get();
}

///////////////////////////////////////////////////////////////////////////////
int CnlMfcThread::Run()
{
  // For worker threads, just call the ThreadHandlerProc,
  // no need to call the base class implementation, which
  // would result in running a messnle pump.
  ASSERT(GetThreadHandler());

  return GetThreadHandler()->ThreadHandlerProc();
}

///////////////////////////////////////////////////////////////////////////////
CnlThreadHandler* CnlMfcThread::GetThreadHandler() const
{
  return Imp()->pThreadHandler.get();
}

