/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlModel.h
//
// @short    See nlModel.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From

//1 Local
#include "nlModel.h"
#include "nlObserver.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::list<CnlObserver*>::const_iterator ObserverIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlModel::SnlImp* CnlModel::Imp()       { return m_pImp; }
inline const  CnlModel::SnlImp* CnlModel::Imp() const { return m_pImp; }

struct CnlModel::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlModel* const pOwner );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlModel* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The list of current observers
  std::list<CnlObserver*> m_observers;

private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlModel::SnlImp::SnlImp(
  CnlModel* const pOwner )
 :
  m_pOwner( pOwner )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlModel::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlModel::CnlModel()
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlModel::~CnlModel()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlModel::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlModel::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlModel::NotifyObservers(
  const int eMessage )
{
  for ( ObserverIter iter = Imp()->m_observers.begin();
        iter != Imp()->m_observers.end();
        ++iter )
  {
    (*iter)->Update( this, eMessage );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlModel::AttachObserver(
  CnlObserver* const pObserver)
{
  Imp()->m_observers.push_back( pObserver );
}

///////////////////////////////////////////////////////////////////////////////
void CnlModel::DetachObserver(
  CnlObserver* const pObserver )
{
  ObserverIter iter = std::find( Imp()->m_observers.begin(), Imp()->m_observers.end(), pObserver );

  if ( iter != Imp()->m_observers.end() )
  {
    //2 Valid observer
    //3 Tell observer to stop observing me
    pObserver->StopObserving( this );
    
    Imp()->m_observers.remove( pObserver );
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlModel::UnregisterAllObservers()
{
}

