//////////////////////////////////////////////////////////////////////////////
//
// @descr    MVC "Model" base class
//
// @short    MVC "Model" base class
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_MODEL_H__
#define __NL_MODEL_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlObserver;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlModel
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlModel();

    // @descr  Post-constructor initialization
    //          Preconditions: None
    //          Postconditions(dependencies):
    //            a. (none) Object safe to use
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    //          Preconditions: None
    //          Postconditions(dependencies):
    //            a. (none) Object safe to delete
    // @end
    void PreDestroy();

    // @descr   Register an Observer to be notified when this model changes
    //          Preconditions:
    //            1. Initialized
    //          Postconditions(dependencies):
    //            a. (1) Observer registered
    // @param   pObserver  IN: Observer to register
    // @end
    void AttachObserver(
      CnlObserver* const pObserver );

    // @descr   Unregister a currently registered Observer
    //          Preconditions:
    //            1. Initialized
    //          Postconditions(dependencies):
    //            a. (1) Observer registered
    // @param   pObserver  IN: Observer to register
    // @end
    void DetachObserver(
      CnlObserver* const pObserver );

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlModel();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

    //1 Final
    // @descr  Notify observers this model has changed
    // @param  eMessage  IN: Message describing the change
    // @end
    void NotifyObservers(
     const int eMessage );

    // @descr  Notify all observers this model is no longer available
    // @end
    void UnregisterAllObservers();

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlModel( const CnlModel& );
    CnlModel& operator=( const CnlModel& );
    CnlModel* operator&();
    const CnlModel* operator&() const;
};

#endif

