//////////////////////////////////////////////////////////////////////////////
//
// @descr    MVC "View" base class
//
// @short    MVC "View" base class
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_OBSERVER_H__
#define __NL_OBSERVER_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlModel;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlObserver
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlObserver();

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlObserver();

    //2 From Cnl

    //1 Abstract
    // @descr   An observed model has changed; update this object accordingly
    //          Preconditions:
    //            1. Initialized
    //          Postconditions(dependencies):
    //            a. (1) Update occurs
    // @param   pModel  IN: Handle to the changed model
    // @param   nChangeType  IN: Change type ID
    // @end
    virtual void Update(
      CnlModel* const pModel,
      const int eChangeType ) = 0;
    
    // @descr   An observed model is no longer available; release handles to it
    //          Preconditions:
    //            1. Initialized
    //          Postconditions(dependencies):
    // @param   Handle to the affectd model
    //            a. (1) Handles to the specified model are released
    // @end
    virtual void StopObserving(
      CnlModel* const pModel ) = 0;

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlObserver( const CnlObserver& );
    CnlObserver& operator=( const CnlObserver& );
};

#endif

