//////////////////////////////////////////////////////////////////////////////
//
// @descr    A container for a variable number of varying types of parameters,
//           currently including bools, numerics and strings.
//
// @short    Variable parameter container.
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_VARIABLE_PARM_CONTAINER_H__
#define __NL_VARIABLE_PARM_CONTAINER_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlVariableParmContainer
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlVariableParmContainer();

    CnlVariableParmContainer( 
      const CnlVariableParmContainer& rhs );
    
    CnlVariableParmContainer& operator=( 
      const CnlVariableParmContainer& rhs );

    void AddBoolParameter(
      const bool bParameter );

    void AddNumericParameter(
      const int nParameter );
    
    void AddStringParameter(
      const CString& strParameter );

    unsigned int GetNumBoolParameters();

    unsigned int GetNumNumericParameters();
    
    unsigned int GetNumStringParameters();
    
    bool GetBoolParameter(
      const unsigned int nParameterOffset );

    int GetNumericParameter(
      const unsigned int nParameterOffset );
    
    CString GetStringParameter(
      const unsigned int nParameterOffset );
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlVariableParmContainer();

    virtual void Dump() const;

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlVariableParmContainer* operator&();
    const CnlVariableParmContainer* operator&() const;
};

#endif

