///////////////////////////////////////////////////////////////////////////////
//
// @file     $Archive: /millennium/src/System/Common/Utilities/Threads/nlWinThread.cpp $
//
// @descr    See nlWinThread.h
//
// @short    See nlWinThread.h
//
// @version  $Revision: 1 $ $Date: 11/11/00 8:49a $ $Author: Spereadwrite $
//
// @author   Doug Robison
//
// @end
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
///////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External

//1 Local
#include "nlMfcThread.h"
#include "nlWinThread.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
//
// Insulated Data
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlWinThread::SnlImp* CnlWinThread::Imp()       { return m_pImp; }
inline const  CnlWinThread::SnlImp* CnlWinThread::Imp() const { return m_pImp; }

struct CnlWinThread::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlWinThread* const pOwner,
    CnlMfcThread* const pMfcThread );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlWinThread* const m_pOwner;

  // The object that creates me
  CnlMfcThread* const m_pMfcThread;
  
  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // Desc

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlWinThread::SnlImp::SnlImp(
  CnlWinThread* const pOwner,
  CnlMfcThread* const pMfcThread )
 :
  m_pOwner( pOwner ),
  m_pMfcThread( pMfcThread )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlWinThread::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlWinThread::CnlWinThread( 
  CnlMfcThread* pMfcThread )
{
  m_pImp = new SnlImp( this, pMfcThread );

  m_bAutoDelete = FALSE;
  m_pMainWnd = NULL;
}

///////////////////////////////////////////////////////////////////////////////
CnlWinThread::~CnlWinThread()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
BOOL CnlWinThread::InitInstance()
{
  ASSERT( Imp()->m_pMfcThread );

  return Imp()->m_pMfcThread->InitInstance();
}

///////////////////////////////////////////////////////////////////////////////
int CnlWinThread::ExitInstance()
{
  ASSERT( Imp()->m_pMfcThread );

  return Imp()->m_pMfcThread->ExitInstance();
}

///////////////////////////////////////////////////////////////////////////////
int CnlWinThread::Run()
{
  ASSERT( Imp()->m_pMfcThread );

  return Imp()->m_pMfcThread->Run();
}

///////////////////////////////////////////////////////////////////////////////
BOOL CnlWinThread::MfcInitInstance()
{
  // we MUST return TRUE here or MFC will shutdown
  // the newly created thread
  return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
int CnlWinThread::MfcExitInstance()
{
  return CWinThread::ExitInstance();
}

///////////////////////////////////////////////////////////////////////////////
int CnlWinThread::MfcRun()
{
  return CWinThread::Run();
}
