//////////////////////////////////////////////////////////////////////////////
//
// @descr    Generic command container
//
// @short    Generic command container
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_COMMAND_H__
#define __NL_COMMAND_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From Utilities
#include <nlVariableParmContainer.h>

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlCommand : public CnlVariableParmContainer
{
  public:
    //1 Final
    // @descr   Constructor
    // @param   eType  IN: The type of this command
    // @end
    explicit CnlCommand(
      const NnlSystemMgrType::EnlCommand eType );

    // @descr   Get the command type
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Type returned
    // @return  Command type
    // @end
    NnlSystemMgrType::EnlCommand GetType() const;

    // @descr   Get a copy of my parms
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Copy returned
    // @return  Parms
    // @end
    CnlVariableParmContainer GetParms() const;

    void Dump() const;

    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlCommand();

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlCommand( const CnlCommand& );
    CnlCommand& operator=( const CnlCommand& );
    CnlCommand* operator&();
    const CnlCommand* operator&() const;
};

#endif

