/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlUserPayload.h
//
// @short    See nlUserPayload.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include "nlCommonType.h"

//1 Local
#include "nlEndPayload.h"
#include "nlSystemMgrType.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::vector< CString >::const_iterator StringParmIter;
  typedef std::vector< long >::const_iterator LongParmIter;
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlEndPayload::SnlImp* CnlEndPayload::Imp()       { return m_pImp; }
inline const  CnlEndPayload::SnlImp* CnlEndPayload::Imp() const { return m_pImp; }

struct CnlEndPayload::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlEndPayload* const pOwner );

  // Copy constructor
  SnlImp( 
    CnlEndPayload* const pOwner,
    const SnlImp& rhs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlEndPayload* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The string parameters for this commnad
  std::vector< CString > m_StringParameters;

  // The numeric parameters for this command
  std::vector< Long > m_NumericParameters;
private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::SnlImp::SnlImp(
  CnlEndPayload* const pOwner )
 :
  m_pOwner( pOwner )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::SnlImp::SnlImp( 
  CnlEndPayload* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( rhs.m_pOwner ),
{
  std::copy( rhs.m_StringParameters.begin(), rhs.m_StringParameters.end(), m_StringParameters );
  std::copy( rhs.m_LongParameters.begin(), rhs.m_LongParameters.end(), m_LongParameters );
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::CnlEndPayload()
{
  m_pImp = new SnlImp( this, 0, NULL );
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::CnlEndPayload(
  const CString& strPayload )
{
  m_pImp = new SnlImp( this, 0, strPayload );
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::CnlEndPayload(
  const long nPayload )
{
  m_pImp = new SnlImp( this, nPayload, NULL );
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::~CnlEndPayload()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload::CnlEndPayload(
  const CnlEndPayload& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlEndPayload& CnlEndPayload::operator=( 
  const CnlEndPayload& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
void CnlEndPayload::SetStringParameter(
  const CString& strParameter )
{
  Imp()->m_StringParameters.push_back( strParameter );
}

///////////////////////////////////////////////////////////////////////////////
unsigned int CnlEndPayload::GeNumStringParameters()
{
  return Imp()->m_StringParameters.size();
}

///////////////////////////////////////////////////////////////////////////////
CString CnlEndPayload::GetStringParameter(
  const unsigned int nParameterId ) const
{
  //1 Error checking
  if ( nParameterId >= Imp()->m_StringParameters.size() )
  {
    return STR_NULL;
  }
  
  return Imp()->m_StringParameters[ nParameterId ];
}


