//////////////////////////////////////////////////////////////////////////////
//
// @descr    Final destination payload.
//
// @short    Final destination payload.
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_END_PAYLOAD_H__
#define __NL_END_PAYLOAD_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlEndPayload
{
  public:
    //1 Final
    // @descr  Default Constructor; creates an invalid payload
    explicit CnlEndPayload();

    // @descr   Set a parameter for this command. Subsequent calls are saved
    //          with sequentially higher parameter IDs, starting with 0.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Parameter saved.
    // @param   nParameter  IN: Numeric parameter for the command
    // @param   strParameter  IN: String parameter for the command
    // @end
    void SetParameter(
      const long nParameter );
    void SetParameter(
      const CString& strParameter );

    // @descr   Get the number of arameters supplied with this command
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Value returned.
    // @return  Number of parameters
    // @end
    unsigned int GetNumParameters(
      const NnlSystemMgrType::EnlParameterType eType ) const;
 
    // @descr   Get the number of string parameters supplied with this command
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Value returned.
    // @return  Number of string parameters
    // @end

    // @descr   Get the parameter
    //          Preconditions:
    //            1. Valid parameter Id (zero-based indexing)
    //          Postconditions(dependencies):
    //            a. (none) Parameter returned; else NULL returned.
    // @param  nParameterId  IN: Number of the parameter
    // @return  Parameter
    // @end
    long GetNumericParameter(
      const unsigned int nParameterId ) const;
    CString GetStringParameter(
      const unsigned int nParameterId ) const;
    
    // @descr   Copy constructor
    // @end
    CnlEndPayload(
      const CnlEndPayload& rhs );

    // @descr  Assignment operator
    // @end
    CnlEndPayload& operator=( 
      const CnlEndPayload& rhs );

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlEndPayload();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlEndPayload* operator&();
    const CnlEndPayload* operator&() const;
};

#endif

