/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlEthernetModule.h
//
// @short    See nlEthernetModule.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>

//1 Local
#include "nlEthernetModule.h"
#include "nlFrame.h"
#include "nlNetworkLayerInterface.h"
#include "nlHwSwInterface.h"
#include "nlPacket.h"
#include "nlCompositeHandle.h"
#include "nlNetworkModel.h"
#include "nlSystemMgr.h"
#include "nlOs.h"
#include "nlNic.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlEthernetModule::SnlImp* CnlEthernetModule::Imp()       { return m_pImp; }
inline const  CnlEthernetModule::SnlImp* CnlEthernetModule::Imp() const { return m_pImp; }

struct CnlEthernetModule::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlEthernetModule* const pOwner,
    CnlOs* const pOs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlEthernetModule* const m_pOwner;

  // The Operating System core
  CnlOs* const m_pOs;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The Packet layer modules I work with
  CnlNetworkLayerInterface* m_pIpModule;
  
private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlEthernetModule::SnlImp::SnlImp(
  CnlEthernetModule* const pOwner,
  CnlOs* const pOs )
 :
  m_pOwner( pOwner ),
  m_pOs( pOs ),
  m_pIpModule( NULL )
//  m_pArpModule( NULL )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlEthernetModule::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlEthernetModule::CnlEthernetModule(
  CnlOs* const pOs )
{
  m_pImp = new SnlImp( this, pOs );
}

///////////////////////////////////////////////////////////////////////////////
CnlEthernetModule::~CnlEthernetModule()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::Initialize(
  CnlNetworkLayerInterface* const pIpModule )
{
  if ( Imp()->m_pIpModule == NULL )
  {
    Imp()->m_pIpModule = pIpModule;
  }
//  if ( Imp()->m_pArpModule == NULL )
//  {
//    Imp()->m_pArpModule = pArpModule;
//  }
}


///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::BroadcastFrame(
  const unsigned int nNicOffset,
  const NnlSystemMgrType::EnlNetworkProtocol eNetworkProtocol,
  const CnlPacket& packet)
{
  //1 Make an Ethernet frame
  const CString strSourceMacAddress =
    Imp()->m_pOs->GetHardwareInterface()->GetNic( nNicOffset )->GetMacAddress();

  CnlFrame frame(
    strSourceMacAddress,
    strEthernetBroadcastAddress, 
    eNetworkProtocol, 
    packet );

  //2 Announce it
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  const CString strProtocol = 
    pNetworkModel->NetworkProtocolToString( eNetworkProtocol );
  CString strMessage;
  strMessage.Format( 
    _T( "Send Ethernet frame: " )
    _T( "Network Protocol: %s, " )
    _T( "Source MAC: %s, " )
    _T( "Destination MAC: %s" ),
    strProtocol,
    strSourceMacAddress,
    strEthernetBroadcastAddress );

  pNetworkModel->SuspendAtPof( 
    Imp()->m_pOs->GetHardwareInterface()->GetProcessingMachineHandle(), 
    LAYER_LINK,
    EVENT_TYPE_SEND,
    strMessage );

  //1 Send it down  
  CnlNic* const pNic = Imp()->m_pOs->GetHardwareInterface()->GetNic( nNicOffset );
  pNic->TransmitFrame( frame );
}

#if 0
///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::BroadcastFrame(
  const unsigned int nNicOffset,
  const NnlSystemMgrType::EnlNetworkProtocol eNetworkProtocol,
  const CnlVariableParmContainer vpc )
{
  //1 Build the frame
  const CString& strSourceMacAddress = 
    Imp()->m_pOs->GetHardwareInterface()->GetMacFromNic( nNicOffset );
  CnlFrame frame(
    strSourceMacAddress,
    strEthernetBroadcastAddress, 
    eNetworkProtocol, 
    vpc );

  //1 Send it down  
  Imp()->m_pOs->GetHardwareInterface()->TransmitFrame( 
   nNicOffset,
   frame );

}

///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::SendFrameToMachine(
  const unsigned int nNicOffset,
  const CString& strDestinationIp,
  const NnlSystemMgrType::EnlNetworkProtocol eNetworkProtocol,
  const CnlVariableParmContainer vpc )
{
  //1 Build the frame
  const CString strDestinationMac = 
    Imp()->m_pIpModule->GetMacFromIp( strDestinationIp );

  if ( strDestinationMac == STR_NULL )
  {
    //3 Don't know where to send this to; abort
    return;
  }

  const CString& strSourceMacAddress = 
    Imp()->m_pOs->GetHardwareInterface()->GetMacFromNic( nNicOffset );

  CnlFrame frame(
    strSourceMacAddress,
    strDestinationMac, 
    eNetworkProtocol, 
    vpc );

  //1 Send it down  
  Imp()->m_pOs->GetHardwareInterface()->TransmitFrame( 
   nNicOffset,
   frame );
}

#endif

///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::SendFrameToMachine(
  const unsigned int nNicOffset,
  const CString& strDestinationMac,
  const NnlSystemMgrType::EnlNetworkProtocol eNetworkProtocol,
  const CnlPacket& packet )
{
  //1 Build the frame
  if ( strDestinationMac == STR_NULL )
  {
    //3 Don't know where to send this to; abort
    return;
  }
  
  const CString& strSourceMacAddress = 
    Imp()->m_pOs->GetHardwareInterface()->GetNic( nNicOffset )->GetMacAddress();

  CnlFrame frame(
    strSourceMacAddress,
    strDestinationMac, 
    eNetworkProtocol, 
    packet );

  //2 Announce it
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  const CString strProtocol = 
    pNetworkModel->NetworkProtocolToString( eNetworkProtocol );
  CString strMessage;
  strMessage.Format( 
    _T( "Send Ethernet frame: " )
    _T( "Network Protocol: %s, " )
    _T( "Source MAC: %s, " )
    _T( "Destination MAC: %s" ),
    strProtocol,
    strSourceMacAddress,
    strDestinationMac );

  pNetworkModel->SuspendAtPof( 
    Imp()->m_pOs->GetHardwareInterface()->GetProcessingMachineHandle(), 
    LAYER_LINK,
    EVENT_TYPE_SEND,
    strMessage );

  //1 Send it down  
  CnlNic* const pNic = Imp()->m_pOs->GetHardwareInterface()->GetNic( nNicOffset );
  pNic->TransmitFrame( frame );
}

///////////////////////////////////////////////////////////////////////////////
void CnlEthernetModule::ReceiveFrame(
  const CnlFrame& frameReceived)
{
  switch( frameReceived.GetNetworkProtocol() )
  {
    case NETWORK_PROTOCOL_IP :
    {
      Imp()->m_pIpModule->ReceivePacket( 
        frameReceived.GetPayload() );
      break;
    }
    case NETWORK_PROTOCOL_ARP:
    {
      //3 Deliver to the ARP module
      Imp()->m_pIpModule->ReceiveArp(
        frameReceived.GetPayload(),
        frameReceived.GetMacAddress( DIRECTION_FROM ) );
      break;
    }
    case NETWORK_PROTOCOL_RARP:
    {
      Imp()->m_pIpModule->ReceiveRarp(
        frameReceived.GetPayload(),
        frameReceived.GetMacAddress( DIRECTION_FROM ) );
      break;
    }
    default : 
    {
      assert( 0 );  // Unknown packet type
    }
  }
}


