//////////////////////////////////////////////////////////////////////////////
//
// @descr    The Ethernet software module
//
// @short    The Ethernet software module
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_ETHERNET_MODULE_H__
#define __NL_ETHERNET_MODULE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From Utilities
#include <nlVariableParmContainer.h>

//1 Local
#include "nlLinkLayerInterface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlHwSwInterface;
class CnlArpModule;
class CnlOs;
class CnlNic;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlEthernetModule : public CnlLinkLayerInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  pOs  IN: The operating system core
    // @end
    CnlEthernetModule(
      CnlOs* const pOs );

    // @descr  Post-constructor initialization
    // @end
    void Initialize(
      CnlNetworkLayerInterface* const pTcpModule );
//      CnlArpModule* const pArpModule );

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    //2 From CnlNetworkLayerInterface
    void BroadcastFrame(
      const unsigned int,
      const NnlSystemMgrType::EnlNetworkProtocol,
      const CnlPacket& );
//    void BroadcastFrame(
//      const unsigned int,
//      const NnlSystemMgrType::EnlNetworkProtocol,
//      const CnlVariableParmContainer );
//    void SendFrameToMachine(
//      const unsigned int,
//      const CString&,
//      const NnlSystemMgrType::EnlNetworkProtocol,
//      const CnlVariableParmContainer );
    void SendFrameToMachine(
      const unsigned int,
      const CString&,
      const NnlSystemMgrType::EnlNetworkProtocol,
      const CnlPacket& );

    void ReceiveFrame(
      const CnlFrame& frame );

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlEthernetModule();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlEthernetModule( const CnlEthernetModule& );
    CnlEthernetModule& operator=( const CnlEthernetModule& );
    CnlEthernetModule* operator&();
    const CnlEthernetModule* operator&() const;
};

#endif

