/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlFrame.h
//
// @short    See nlFrame.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlFrame.h"
#include "nlPacket.h"
#include "nlSystemMgrType.h"
#include "nlCommonType.h"
#include "nlVariableParmContainer.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlFrame::SnlImp* CnlFrame::Imp()       { return m_pImp; }
inline const  CnlFrame::SnlImp* CnlFrame::Imp() const { return m_pImp; }

struct CnlFrame::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlFrame* const pOwner,
    const CString strFromMacAddress,
    const CString strToMacAddress,
    const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
    CnlPacket packet,
    const CnlVariableParmContainer& vpc );

  // Copy constructor
  SnlImp( 
    CnlFrame* const pOwner,
    const SnlImp& rhs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlFrame* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The "from" MAC address field
  const CString m_strFromMacAddress;
  
  // The "to" MAC address field
  const CString m_strToMacAddress;
  
  // The type of my payload
  const NnlSystemMgrType::EnlNetworkProtocol m_ePacketType;

  // The upper-layer packet I contain, if any
  const CnlPacket m_Packet;

  // The VPC payload I contain, if any
  const CnlVariableParmContainer m_Vpc;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlFrame::SnlImp::SnlImp(
  CnlFrame* const pOwner,
  const CString strFromMacAddress,
  const CString strToMacAddress,
  const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
  CnlPacket packet,
  const CnlVariableParmContainer& vpc )
 :
  m_pOwner( pOwner ),
  m_strFromMacAddress( strFromMacAddress ),
  m_strToMacAddress( strToMacAddress ),
  m_ePacketType( ePacketType ),
  m_Packet( packet ),
  m_Vpc( vpc )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlFrame::SnlImp::SnlImp( 
  CnlFrame* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( pOwner ),
  m_ePacketType( rhs.m_ePacketType ),
  m_Packet( rhs.m_Packet ),
  m_Vpc( rhs.m_Vpc )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlFrame::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlFrame::CnlFrame()
{
  m_pImp = new SnlImp( 
    this, 
    _T( "00:00:00:00:00:00" ),
    _T( "00:00:00:00:00:00" ),
    NETWORK_PROTOCOL_UNKNOWN, 
    CnlPacket(),
    CnlVariableParmContainer() );
}

///////////////////////////////////////////////////////////////////////////////
CnlFrame::CnlFrame(
  const CString strFromMacAddress,
  const CString strToMacAddress,
  const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
  const CnlPacket& packet )
{
  m_pImp = new SnlImp( 
    this, 
    strFromMacAddress,
    strToMacAddress,
    ePacketType, 
    packet,
    CnlVariableParmContainer() );  // Empty container
}

#if 0
///////////////////////////////////////////////////////////////////////////////
CnlFrame::CnlFrame(
  const CString strFromMacAddress,
  const CString strToMacAddress,
  const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
  const CnlVariableParmContainer& vpc )
{
  m_pImp = new SnlImp( 
    this, 
    strFromMacAddress,
    strToMacAddress,
    ePacketType,
    CnlPacket(),  // Empty packet
    vpc );
}
#endif

///////////////////////////////////////////////////////////////////////////////
CnlFrame::~CnlFrame()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlFrame::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlFrame::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
CnlFrame::CnlFrame(
  const CnlFrame& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlFrame& CnlFrame::operator=( 
  const CnlFrame& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlFrame::GetMacAddress(
  const EnlDirection eDirection ) const
{
  switch( eDirection )
  {
    case DIRECTION_FROM :
    {
      return Imp()->m_strFromMacAddress;
    }
    case DIRECTION_TO :
    {
      return Imp()->m_strToMacAddress;
    }
    default : assert( 0 );  // Don't know this type
  }

  return STR_NULL;
}

///////////////////////////////////////////////////////////////////////////////
EnlLinkProtocol CnlFrame::GetFrameType() const
{
  return LINK_PROTOCOL_ETHERNET;
}

///////////////////////////////////////////////////////////////////////////////
EnlNetworkProtocol CnlFrame::GetNetworkProtocol() const
{
  return Imp()->m_ePacketType;
}

///////////////////////////////////////////////////////////////////////////////
EnlLinkProtocol CnlFrame::GetLinkProtocol() const
{
  return LINK_PROTOCOL_ETHERNET;
}


///////////////////////////////////////////////////////////////////////////////
CnlPacket CnlFrame::GetPayload() const
{
  return Imp()->m_Packet;
}

#if 0
///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer CnlFrame::GetVpcPayload() const
{
  return Imp()->m_Vpc;
}

#endif
