//////////////////////////////////////////////////////////////////////////////
//
// @descr    Link-layer frame
//
// @short    Link-layer frame
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_FRAME_H__
#define __NL_FRAME_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlPacket;
class CnlVariableParmContainer;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlFrame
{
  public:
    //1 Final
    // @descr  Default Constructor
    // @end
    explicit CnlFrame();

    // @descr  Constructor
    // @param  strFromMacAddress  IN: Mac address of the sending Nic
    // @param  strToMacAddress  IN: Mac address of the target Nic
    // @param  ePacketType  IN: The type of packet in my payload
    // @param  pPacket or vpc  IN: My payload
    // @end
    explicit CnlFrame(
      const CString strFromMacAddress,
      const CString strToMacAddress,
      const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
      const CnlPacket& pPacket );
//    explicit CnlFrame(
//      const CString strFromMacAddress,
//      const CString strToMacAddress,
//      const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
//      const CnlVariableParmContainer& vpc );


    // @descr  Copy constructor
    // @end
    CnlFrame(
      const CnlFrame& rhs );

    // @descr  Assignment operator
    // @end
    CnlFrame& operator=(
      const CnlFrame& rhs );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Get one of the MAC addresses in the header
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Address returned
    // @param   eDirection  IN: Which address to extract
    // @return  Address
    // @end
    CString GetMacAddress(
      const NnlSystemMgrType::EnlDirection eDirection ) const;


    // @descr   Get the frame type. 
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Type returned. Currently, this is hardcoded to Ethernet.
    // @return  Type
    // @end
    NnlSystemMgrType::EnlLinkProtocol GetFrameType() const;

    // @descr   Get the frame type. 
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Type returned.
    // @return  Type
    // @end
    NnlSystemMgrType::EnlNetworkProtocol GetNetworkProtocol() const;

    // @descr   Get the encoding scheme to use. 
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Type returned. Currently, this is hardcoded to Ethernet.
    // @return  Type
    // @end
    NnlSystemMgrType::EnlLinkProtocol GetLinkProtocol() const;

    // @descr   Extract the packet payload
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Packet extracted
    // @return  Packet
    // @end
    CnlPacket GetPayload() const;
//    CnlVariableParmContainer GetVpcPayload() const;


    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlFrame();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlFrame* operator&();
    const CnlFrame* operator&() const;
};

#endif

