//////////////////////////////////////////////////////////////////////////////
//
// @descr    The interface between hardware and software entities
//
// @short    The interface between hardware and software entities
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_HW_SW_INTERFACE_H__
#define __NL_HW_SW_INTERFACE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlFrame;
class CnlCompositeHandle;
class CnlNic;
class CnlOs;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlHwSwInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @param   nX  IN: Desc
    //            (cont'd)
    // @end
    explicit CnlHwSwInterface()
    {}

    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlHwSwInterface()
    {}

    // @descr   Identifies if this machine is turned on.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned.
    // @return  Current power status
    // @end
    virtual bool IsMachineOn() const = 0;
    
    //1 Abstract
    // @descr   Get the number of NICs installed
    //          Preconditions:
    //            1. This is an End Machine
    //            2. This is a Router
    //          Postconditions(dependencies):
    //            a. (1) The total number of NICs is returned
    //            b. (2) The total number of client-serving NICs is returned
    // @end
    virtual unsigned int GetNumNics() const = 0;

    // @descr   Connect to the specified NIC
    //          Preconditions:
    //            1. NIC exists
    //          Postconditions(dependencies):
    //            a. (all) Pointer returned; else NULL returned
    // @param   nNicId  IN: Zero based card offset
    // @return  NIC connection
    // @end
    virtual CnlNic* GetNic(
      const unsigned int nNicId ) const = 0;

    // @descr   Get the Operating System on this machine
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) OS connection returned
    // @return  OS connection
    // @end
    virtual CnlOs* GetOs() const = 0;

    // @descr   Get the handle used by the End Machine
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Handle returned
    // @return  Handle
    // @end
    virtual CnlCompositeHandle GetProcessingMachineHandle() const = 0;

  protected:

  private:

    //1 Implementation
    //3 None
    
    //1 Unimplemented
    CnlHwSwInterface( const CnlHwSwInterface& );
    CnlHwSwInterface& operator=( const CnlHwSwInterface& );
    CnlHwSwInterface* operator&();
    const CnlHwSwInterface* operator&() const;
};

#endif

