//////////////////////////////////////////////////////////////////////////////
//
// @descr    A network address, stored in both classles string and long formats.
//
// @short    Network address
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_IPV4_ADDRESS_H__
#define __NL_IPV4_ADDRESS_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgr.h"
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlIPv4Address
{
  public:

    //1 Final
    // @descr   Default constructor
    // @end
    explicit CnlIPv4Address();

    // @descr   Constructor
    //          Preconditions:
    //            1. Valid format
    //          Postconditions(dependencies):
    //            a. (all) Valid object created; else default object created
    // @param   strClasslessAddress  IN: Address in classless format
    // @end
    explicit CnlIPv4Address(
      const CString& strClasslessAddress );

    // Copy constructor
    CnlIPv4Address(
      const CnlIPv4Address& rhs );

    // Assignment operator
    CnlIPv4Address& operator=(
      const CnlIPv4Address& rhs );

    // Equivalence operator
    bool operator==(
      const CnlIPv4Address& rhs ) const;

    // Inequality operator
    bool operator!=(
      const CnlIPv4Address& rhs ) const;

    // Less than operator
    bool operator<(
      const CnlIPv4Address& rhs ) const;

    // @descr   Describes the validity of this object
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Result returned
    // @return  State
    // @end
    NnlSystemMgrType::EnlResult Validity();

    // @descr   Get the classless string-formatted version
    //          Preconditions:
    //            1. Valid object
    //          Postconditions(dependencies):
    //            a. (all) Address returned; else NULL
    // @return  Address or NULL
    // @end
    CString Classless() const;

    // @descr   Get the classfull network address in dot notation
    //          Preconditions:
    //            1. Valid object
    //          Postconditions(dependencies):
    //            a. (all) Address returned; else NULL
    // @return  Address or NULL
    // @end
    CString Network() const;

    // @descr   Get the classfull machine address in dot notation
    //          Preconditions:
    //            1. Valid object
    //          Postconditions(dependencies):
    //            a. (all) Address returned; else NULL
    // @return  Address or NULL
    // @end
    CString Machine() const;

    // @descr   Get the broadcast address in dot notation
    //          Preconditions:
    //            1. Valid object
    //          Postconditions(dependencies):
    //            a. (all) Address returned; else NULL
    // @return  Address or NULL
    // @end
    CString Broadcast() const;

    // @descr   Get the gateway address in dot notation
    //          Preconditions:
    //            1. Valid object
    //          Postconditions(dependencies):
    //            a. (all) Address returned; else NULL
    // @return  Address or NULL
    // @end
    CString Gateway() const;

    // @descr   Get the number of significant places in the address in bits
    //          Preconditions:
    //            1. Valid object
    //          Postconditions(dependencies):
    //            a. (all) Number of places returned; else -1
    // @return  Places or -1
    // @end
    int Length() const;
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlIPv4Address();

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlIPv4Address* operator&();
    const CnlIPv4Address* operator&() const;
};

#endif

