//////////////////////////////////////////////////////////////////////////////
//
// @descr    The Internet Protocol software module for routers
//
// @short    The Internet Protocol software module for routers
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_IP_MODULE_ROUTER_H__
#define __NL_IP_MODULE_ROUTER_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlIpModule.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlPacket;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlIpModuleRouter : public CnlIpModule
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  pOs  IN: The operating system core
    // @end
    CnlIpModuleRouter(
      CnlOs* const pOs );

    void Initialize(
//      CnlArpModule* const pArpModule,
      CnlLinkLayerInterface* const pEthernetModule );

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    //2 From CnlIpModule

    //2 From CnlNetworkLayerInterface
    bool ReceivePacket(
      const CnlPacket& packet );
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlIpModuleRouter();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

    //1 Final
    //2 From CnlIpModule
    void FillRoutingTable();
    void QueryForRouterIp();

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlIpModuleRouter( const CnlIpModuleRouter& );
    CnlIpModuleRouter& operator=( const CnlIpModuleRouter& );
    CnlIpModuleRouter* operator&();
    const CnlIpModuleRouter* operator&() const;
};

#endif

