/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlLink.h
//
// @short    See nlLink.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCompositeHandle.h>
//#include <agMfcWorkerThread.h>
//#include <agThreadHandler.h>

//1 Local
#include "nlLink.h"
#include "nlNic.h"
#include "nlSystemMgrType.h"
#include "nlSystemMgr.h"
#include "nlNetworkModel.h"
#include "nlFrame.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include <vector>

using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
  typedef std::vector< CnlNic* >::iterator NicIter;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Worker thread classes
//
///////////////////////////////////////////////////////////////////////////////

#if 0
///////////////////////////////////////////////////////////////////////////////
class CnlExecuteCommandThread : public CagMfcWorkerThread
{
  public:

    // @descr  Constructor
    // @param  pNetworkModel  IN: Handle to the Network Model
    // @end
    CnlExecuteCommandThread(
      CnlLink* const pLink,
      const unsigned int nConnection );
};

class CnlExecuteCommandThreadHandler : public CagThreadHandler
{
  public:

    // @descr  Constructor
    // @param  pNetworkModel  IN: Handle to the Network Model
    // @end
    CnlExecuteCommandThreadHandler(
      CnlLink* const pLink,
      const unsigned int nConnection );

    // @descr   Called when the threads starts executing.
    // @return  
    // @end
    //
    unsigned __int32 ThreadHandlerProc();

  private:

    // Handle to the Network Model
    CnlLink* const m_pLink;
    const unsigned int m_nConnection;
};
#endif 

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlLink::SnlImp* CnlLink::Imp()       { return m_pImp; }
inline const  CnlLink::SnlImp* CnlLink::Imp() const { return m_pImp; }

struct CnlLink::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlLink* const pOwner,
    CnlCompositeHandle hNetwork );

  // Destructor
   ~SnlImp();

//  NnlSystemMgrType::EnlResult Relay(
//    const unsigned int nConnection );
  
  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlLink* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The user's handle to the network I am a part of
  const CnlCompositeHandle m_hThis;
  
  // The collection of Nic cards that are connected
  std::vector< CnlNic* > m_Nics;

  // Handle to the threads I'm currently running
//  std::vector< HANDLE > m_ThreadHandles;

  // The return value from the threaded ExecuteCommand call
//  std::vector< NnlSystemMgrType::EnlResult > m_ThreadResults;

  // The thread which handles command execution
//  std::vector< CnlExecuteCommandThread* > m_ExecuteCommandThreads;

  // The frame I'm currently transmitting
//  CnlFrame m_frame;
  
private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlLink::SnlImp::SnlImp(
  CnlLink* const pOwner,
  CnlCompositeHandle hNetwork )
 :
  m_pOwner( pOwner ),
  m_hThis( hNetwork )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlLink::SnlImp::~SnlImp()
{
}

#if 0
///////////////////////////////////////////////////////////////////////////////
NnlSystemMgrType::EnlResult CnlLink::SnlImp::Relay(
  const unsigned int nConnection )
{
  CnlNic* const pNic = m_Nics[ nConnection ];
  const CString& strMac = pNic ->GetMacAddress();

  //1 Announce it
  CString strMessage;
  strMessage.Format( 
    _T( "Link relaying frame to %s" ),
    strMac );

  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  pNetworkModel->SuspendAtPof( 
    Imp()->m_hThis, 
    LAYER_PHYSICAL,
    false,
    strMessage );

  //1 Do it
  pNic ->ReceiveFrame( frame );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNetworkModel::SuspendAtPof(
  const CnlCompositeHandle hPof,
  const NnlSystemMgrType::EnlLayer eLayer,
  const bool bIsRejection,
  const CString& strMessage )
{
  //1 Check filters
  if ( eLayer < Imp()->m_eLowestLayerToShow )
  {
    return RESULT_LAYER_FILTERED_OUT;
  }

  if ( bIsRejection && ! Imp()->m_bShowRejectionMessages )
  {
    return RESULT_EVENT_TYPE_FILTERED_OUT;
  }

  //1 Do it
  Imp()->m_hPof = hPof;
  Imp()->m_strPofMessage = strMessage;
  const CString strDebug = STR_NEWLINE  + _T( "POF: " + Imp()->m_strPofMessage );
  TRACE0( strDebug );
  Imp()->m_eMessageFromThread = MESSAGE_POF_CHANGED;
  ::SuspendThread( Imp()->m_hThisThread );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
CnlExecuteCommandThread::CnlExecuteCommandThread( 
  CnlLink* const pLink,
  const unsigned int nConnection )
 :
  CagMfcWorkerThread(  new CnlExecuteCommandThreadHandler(
    pLink,
    nConnection ),
    CREATE_SUSPENDED )
{
  pLink->Imp()->m_ThreadHandles[ nConnection ] = GetHandle();
  SetThreadPriority( GetHandle(), THREAD_PRIORITY_BELOW_NORMAL );
  ::ResumeThread( GetHandle() );
}

///////////////////////////////////////////////////////////////////////////////
CnlExecuteCommandThreadHandler::CnlExecuteCommandThreadHandler(
  CnlLink* const pLink,
  const unsigned int nConnection )
 : 
  m_pLink( pLink ),
  m_nConnection( nConnection )
{
}

///////////////////////////////////////////////////////////////////////////////
unsigned __int32 CnlExecuteCommandThreadHandler::ThreadHandlerProc()
{
  TRACE1( _T( "\n=====START OF RELAY #%d=====\n" ), m_nConnection );

  m_pLink->Imp()->m_ThreadResults[ m_nConnection ] = 
    m_pLink->Imp()->Relay( m_nConnection );

  TRACE1( _T( "\n=====END OF RELAY #%d=====\n" ), m_nConnection );
  return 0;
}
#endif 

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlLink::CnlLink(
  CnlCompositeHandle hNetwork )
{
  m_pImp = new SnlImp( this, hNetwork );
}

///////////////////////////////////////////////////////////////////////////////
CnlLink::~CnlLink()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlLink::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlLink::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlLink::ConnectNic(
  CnlNic* const pNic )
{
  Imp()->m_Nics.push_back( pNic );
  const EnlResult eResult = pNic->ConnectLink( this );
  assert( eResult == RESULT_OK );
}

///////////////////////////////////////////////////////////////////////////////
void CnlLink::DisconnectNic(
  CnlNic* const pNic )
{
  //!!!todo
}

///////////////////////////////////////////////////////////////////////////////
void CnlLink::RelayFrame(
  const CnlFrame& frame )
{
  //1 Announce it
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  const CString strProtocol = 
    pNetworkModel->LinkProtocolToString( frame.GetLinkProtocol());
  CString strMessage;
  strMessage.Format( 
    _T( "Sending bits:" )
    _T( "Link Protocol: %s" ),
    strProtocol );

  pNetworkModel->SuspendAtPof( 
    Imp()->m_hThis, 
    LAYER_PHYSICAL,
    EVENT_TYPE_SEND,
    strMessage );

  for ( NicIter iter = Imp()->m_Nics.begin();
        iter != Imp()->m_Nics.end();
        ++iter )
  {
    CnlNic* const pNic = *iter;
    const CString& strMac = pNic ->GetMacAddress();
    
    //1 Do it
    pNic ->ReceiveFrame( frame );
  }
}


