//////////////////////////////////////////////////////////////////////////////
//
// @descr    A datalink connecting two nodes
//
// @short    A datalink connecting two nodes
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_LINK_H__
#define __NL_LINK_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From Utilities
#include <nlCompositeHandle.h>

// Local
#include "nlNetworkLeaf.h"
#include "nlNic.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlLink : public CnlNetworkLeaf
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  The user's handle to the network I am a part of
    // @end
    explicit CnlLink(
      CnlCompositeHandle hNetwork );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Connect the specified Nic to this link.
    //          Preconditions: None
    //          Postconditions(dependencies):
    //            a. (none) Nic connected
    // @param   pNic  IN: Handle to the Nic to connect
    // @end
    void ConnectNic(
      CnlNic* const pNic );

    // @descr   Disconnect the specified Nic to this link.
    //          Preconditions: None
    //          Postconditions(dependencies):
    //            a. (none) Nic connected
    // @param   pNic  IN: Handle to the Nic to connect
    // @end
    void DisconnectNic(
      CnlNic* const pNic );

    // @descr   RelayFrame a command and its parameter to all connected Nics
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Command relayed to all connected Nics
    // @param   frame  IN: Frame to relay
    // @end
    void RelayFrame(
//      NnlSystemMgrType::EnlCommand nCommand,
//      const CString& strParameter );
      const CnlFrame& frame );

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlLink();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    friend class CnlExecuteCommandThread;
    friend class CnlExecuteCommandThreadHandler;
    
    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlLink( const CnlLink& );
    CnlLink& operator=( const CnlLink& );
    CnlLink* operator&();
    const CnlLink* operator&() const;
};

#endif

