//////////////////////////////////////////////////////////////////////////////
//
// @descr    Interface for Link Layer modules
//
// @short    Interface for Link Layer modules
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_LINK_LAYER_INTERFACE_H__
#define __NL_LINK_LAYER_INTERFACE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlNetworkLayerInterface;
class CnlPacket;
class CnlFrame;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlLinkLayerInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlLinkLayerInterface()
    {}

    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlLinkLayerInterface()
    {}

    //1 Abstract
    // @descr   Build a frame out of a packet and send it to 
    //          all machines in the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload passed down in a segment
    // @param   segment  IN: Payload to deliver to Link layer
    // @end
    virtual void BroadcastFrame(
      const unsigned int nNicOffset,
      const NnlSystemMgrType::EnlNetworkProtocol ePacketType,
      const CnlPacket& packet ) = 0;

    // @descr   Build a frame out of a generic payload and send it to 
    //          all machines in the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload broadcast
    // @param   eType  IN: Type of payload
    // @param   vpc  IN: Payload
    // @end
//    virtual void BroadcastFrame(
//      const unsigned int nNicOffset,
//      const NnlSystemMgrType::EnlNetworkProtocol eType,
//      const CnlVariableParmContainer vpc ) = 0;

    // @descr   Build a frame out of a generic payload and send it to 
    //          the specified machine in the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload broadcast
    // @param   nNicOffset  IN: NIC to use
    // @param   strDestinationIp  IN: Destination IP address
    // @param   eType  IN: Type of payload
    // @param   vpc or packet  IN: Payload
    // @end
//    virtual void SendFrameToMachine(
//      const unsigned int nNicOffset,
//      const CString& strDestinationIp,
//      const NnlSystemMgrType::EnlNetworkProtocol eType,
//      const CnlVariableParmContainer vpc ) = 0;
    virtual void SendFrameToMachine(
      const unsigned int nNicOffset,
      const CString& strDestinationIp,
      const NnlSystemMgrType::EnlNetworkProtocol eType,
      const CnlPacket& packet ) = 0;

    // @descr   Receive a frame from the network.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Frame processed
    // @param   frame  IN: Frame to process.
    // @end
    virtual void ReceiveFrame(
      const CnlFrame& frame ) = 0;

  protected:

  private:

    //1 Implementation
    //3 None
    
    //1 Unimplemented
    CnlLinkLayerInterface( const CnlLinkLayerInterface& );
    CnlLinkLayerInterface& operator=( const CnlLinkLayerInterface& );
    CnlLinkLayerInterface* operator&();
    const CnlLinkLayerInterface* operator&() const;
};

#endif

