//////////////////////////////////////////////////////////////////////////////
//
// @descr    A subnet, network or internet
//
// @short    A subnet, network or internet
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_NETWORK_H__
#define __NL_NETWORK_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local
#include "nlNetworkComposite.h"
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlProcessingMachine;
class CnlCompositeHandle;
class CnlCommand;
class CnlIPv4Address;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlNetwork : public CnlNetworkComposite
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  hThis  IN: Handle to this object
    // @param  address  IN: Address to assign to this network
    // @end
    CnlNetwork(
      CnlCompositeHandle hThis,
      const CnlIPv4Address& address );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Add a new end machine to this network
    //          Preconditions:
    //            1. There is at least 1 available address
    //          Postconditions(dependencies):
    //            a. (none) End machine added
    // @param   hProcessingMachine  OUT: Handle to the machine created
    // @param   hNetwork  IN: Handle to owning network
    // @return  True if postcondition statisfied; False otherwise
    // @end
    NnlSystemMgrType::EnlResult CreateProcessingMachine(
      CnlCompositeHandle& hProcessingMachine,
      CnlCompositeHandle hNetwork );

    // @descr   Destroy the specified end machine.
    //          Preconditions:
    //            1. Machine exists
    //          Postconditions(dependencies):
    //            a. (all) Machine destroyed
    // @param   hProcessingMachine  IN: Handle to the machine to destroy
    // @return  Request result
    // @end
//    NnlSystemMgrType::EnlResult DestroyProcessingMachine(
//      CnlCompositeHandle hProcessingMachine);

    // @descr   Connect the specified router's NIC to my link
    //          Preconditions:
    //            1. NIC exists
    //          Postconditions(dependencies):
    //            a. (all) Connection made
    // @param   pRouter  IN: Router to connect to
    // @param   nNicId  IN: NIC of the router to use
    // @return  Request result
    // @end
    NnlSystemMgrType::EnlResult ConnectRouter(
      const CnlProcessingMachine* pRouter,
      const unsigned int nNicId );

    //2 From Cnl

    //2 Forwarded by CnlNetworkModel
    NnlSystemMgrType::EnlResult AssignIpAddress(
      const CnlCompositeHandle,
      const unsigned int,
      const CString& );
    NnlSystemMgrType::EnlResult ExecuteCommand(
      const CnlCompositeHandle,
      const CnlCommand* const );
    NnlSystemMgrType::EnlResult PostExecuteCommand(
      const CnlCompositeHandle,
      const CnlCommand* const );
    NnlSystemMgrType::EnlResult GetProcessingMachineIpAddress(
      CString&,
      const CnlCompositeHandle,
      const unsigned int ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineMacAddress(
      CString&,
      const CnlCompositeHandle,
      const unsigned int ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineNicCount(
      unsigned int&,
      const CnlCompositeHandle ) const;
    NnlSystemMgrType::EnlResult GetProcessingMachineString(
      CString&,
      const CnlCompositeHandle,
      NnlSystemMgrType::EnlCommandLineEntry ) const;
    NnlSystemMgrType::EnlResult ToggleProcessingMachinePower(
      const CnlCompositeHandle );
    NnlSystemMgrType::EnlResult GetProcessingMachinePowerStatus(
      bool&,
      const CnlCompositeHandle ) const;
    NnlSystemMgrType::EnlResult ClearProcessingMachineArpCache(
      const CnlCompositeHandle hProcessingMachine );
  
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlNetwork();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlNetwork( const CnlNetwork& );
    CnlNetwork& operator=( const CnlNetwork& );
    CnlNetwork* operator&();
    const CnlNetwork* operator&() const;
};

#endif

