/////////////////////////////////////////////////////////////////////////////
//
// @descr    CnlNetworkModel Unit Tests
//
// @short    CnlNetworkModel Unit Tests
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utils
#include <nlCommonType.h>

//1 Local
#include "nlNetworkModel.h"
#include "nlNetworkModelImp.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//using namespace xx;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
bool CnlNetworkModel::SnlImp::UnitTest()
{
  bool bResult = false;

  //1 Test ParseClasslessFormat()
  CString strParsedAddress;
  int nParsedLength;

  //2 Test good formats
  bResult = ParseClasslessFormat(
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/1" ) );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  //2 Test bad formats
  bResult = ParseClasslessFormat(
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0" ) );

  if ( bResult )
  {
    // False positive
    return false;
  }

  bResult = ParseClasslessFormat(
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/1/2" ) );

  if ( bResult )
  {
    // False positive
    return false;
  }

  //1Test ValidateSubnetAddressFormat()

  //2 Test good formats
  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/1" ) );

  bResult = m_pOwner->ValidateNetworkAddressString( strParsedAddress );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "255.255.255.255/1" ) );

  bResult = m_pOwner->ValidateNetworkAddressString( strParsedAddress );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  //2 Test bad formats
  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "-1.0.0.0/1" ) );

  bResult = m_pOwner->ValidateNetworkAddressString( strParsedAddress );

  if ( bResult )
  {
    // False positive
    return false;
  }

  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "256.0.0.0/1" ) );

  bResult = m_pOwner->ValidateNetworkAddressString( strParsedAddress );

  if ( bResult )
  {
    // False positive
    return false;
  }


  //1Test ValidateSubnetAddressLength()

  //2 Test good formats
  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/24" ) );

  bResult = ValidateSubnetAddressLength( nParsedLength );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/30" ) );

  bResult = ValidateSubnetAddressLength( nParsedLength );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  //2 Test bad formats
  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/23" ) );

  bResult = ValidateSubnetAddressLength( nParsedLength );

  if ( bResult )
  {
    // False positive
    return false;
  }

  strParsedAddress = NnlType::STR_NULL;
  nParsedLength = 0;
  bResult = ParseClasslessFormat( 
    strParsedAddress,
    nParsedLength,
    _T( "0.0.0.0/31" ) );

  bResult = ValidateSubnetAddressLength( nParsedLength );

  if ( bResult )
  {
    // False positive
    return false;
  }

  //1 Test IsSubnetAddressUnique()_

  //2 Test good values
  m_NetworkAddressRegistry.clear();
  bResult = IsSubnetAddressUnique( _T( "0.0.0.0" ) );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  //2 Test bad values
  m_NetworkAddressRegistry.insert( _T( "0.0.0.0" ) );
  bResult = IsSubnetAddressUnique( _T( "0.0.0.0" ) );

  if ( bResult )
  {
    // False positive
    return false;
  }

  m_NetworkAddressRegistry.clear();


  //1 Test IsNumProcessingMachinesValidForSubnet()

  //2 Test good values
  bResult = IsNumProcessingMachinesValidForSubnet( 29, 1 );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  bResult = IsNumProcessingMachinesValidForSubnet( 29, 5 );

  if ( ! bResult )
  {
    // False negative
    return false;
  }

  //2 Test bad values
  bResult = IsNumProcessingMachinesValidForSubnet( 29, 0 );

  if ( bResult )
  {
    // False positive
    return false;
  }

  bResult = IsNumProcessingMachinesValidForSubnet( 29, 6 );

  if ( bResult )
  {
    // False positive
    return false;
  }

  //1 All tests passed
  return true;
}
///////////////////////////////////////////////////////////////////////////////
