/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlNic.h
//
// @short    See nlNic.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>

//1 Local
#include "nlNic.h"
#include "nlSystemMgr.h"
#include "nlLink.h"
#include "nlProcessingMachine.h"
#include "nlFrame.h"
#include "nlSystemMgr.h"
#include "nlNetworkModel.h"
#include "nlOs.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlNic::SnlImp* CnlNic::Imp()       { return m_pImp; }
inline const  CnlNic::SnlImp* CnlNic::Imp() const { return m_pImp; }

struct CnlNic::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlNic* const pOwner,
    CnlHwSwInterface* const pProcessingMachine );


  // Destructor
   ~SnlImp();

  bool IsForMe(
    const CnlFrame& frame ) const;

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlNic* const m_pOwner;

  // The end machine I am inside
  CnlHwSwInterface* const m_pHwSwInterface;
  
  // The link I am connected to
  CnlLink* m_pLink;

  // The Ethernet module in my Os
  CnlLinkLayerInterface* m_pEthernetModule;
  
  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // MAC address burned into this card
  const CString m_strMacAddress;
  
private:

  // Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlNic::SnlImp::SnlImp(
  CnlNic* const pOwner,
  CnlHwSwInterface* const pProcessingMachine )
 :
  m_pOwner( pOwner ),
  m_pLink( NULL ),
  m_pHwSwInterface( pProcessingMachine ),
  m_pEthernetModule( NULL ),
  m_strMacAddress( CnlSystemMgr::Instance().GetInternalInterface()->MakeMacAddress() )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlNic::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
bool CnlNic::SnlImp::IsForMe(
    const CnlFrame& frame ) const
{
  const bool bIsForMe = 
    (  ( frame.GetMacAddress( DIRECTION_TO ) == strEthernetBroadcastAddress )
    || frame.GetMacAddress( DIRECTION_TO ) == m_strMacAddress );
  return bIsForMe;
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlNic::CnlNic(
  CnlHwSwInterface* const pProcessingMachine )
{
  m_pImp = new SnlImp( this, pProcessingMachine );
}

///////////////////////////////////////////////////////////////////////////////
CnlNic::~CnlNic()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNic::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlNic::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlNic::SetEthernetModule(
  CnlLinkLayerInterface* const pEthernetModule )
{
  if ( Imp()->m_pEthernetModule == NULL )
  {
    Imp()->m_pEthernetModule = pEthernetModule;
  }
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlNic::ConnectLink(
  CnlLink* const pLink)
{
  //1 Error checking
  if ( Imp()->m_pLink != NULL )
  {
    //3 Already connected to a link
    return RESULT_ERROR_ALREADY_CONNECTED;
  }
    
  Imp()->m_pLink = pLink;
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
void CnlNic::TransmitFrame(
  const CnlFrame& frame )
{
  if ( ! Imp()->m_pHwSwInterface->IsMachineOn() )
  {
    return;
  }
  
  Imp()->m_pLink->RelayFrame( frame );
}

///////////////////////////////////////////////////////////////////////////////
void CnlNic::ReceiveFrame(
  const CnlFrame& frame )
{
  if ( ! Imp()->m_pHwSwInterface->IsMachineOn() )
  {
    return;
  }
  
  //1 Pass this frame up as appropriate
  CString strReason = STR_NULL;

  if ( ! Imp()->IsForMe( frame ) )
  {
    return;
  }

  //2 Announce it
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  const CString strProtocol = 
    pNetworkModel->NetworkProtocolToString( frame.GetNetworkProtocol() );
  CString strMessage;
  strMessage.Format( 
    _T( "Receiving Ethernet frame: " )
    _T( "Network Protocol: %s, " )
    _T( "Source MAC: %s, " )
    _T( "Destination MAC: %s" ),
    strProtocol,
    frame.GetMacAddress( DIRECTION_FROM ),
    frame.GetMacAddress( DIRECTION_TO ) );

  pNetworkModel->SuspendAtPof( 
    Imp()->m_pHwSwInterface->GetProcessingMachineHandle(), 
    LAYER_LINK,
    EVENT_TYPE_RECEIVE_ACCEPT,
    strMessage );

  //2 Do it
  Imp()->m_pHwSwInterface->GetOs()->ReceiveFrame( frame );
}

///////////////////////////////////////////////////////////////////////////////
CString CnlNic::GetMacAddress() const
{
  return Imp()->m_strMacAddress;
}

