//////////////////////////////////////////////////////////////////////////////
//
// @descr    A network interface card residing in an end machine
//
// @short    A network interface card residing in an end machine
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_NIC_H__
#define __NL_NIC_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlLink;
class CnlHwSwInterface;
class CnlLinkLayerInterface;
class CnlFrame;
class CnlCompositeHandle;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlNic
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  pProcessingMachine  IN: Connection to the end machine I am inside
    // @end
    explicit CnlNic(
      CnlHwSwInterface* const pProcessingMachine );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Connect to a link.
    //          Preconditions:
    //            1. Not already connected to a link.
    //          Postconditions(dependencies):
    //            a. (all) Connection made
    // @param   pLink  IN: Link to connect to.
    // @return  Request result
    // @end
    NnlSystemMgrType::EnlResult ConnectLink(
      CnlLink* const pLink);

    // @descr   Set the connection to the ethernet driver.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Connection made.
    // @param   pEthernetModule  IN: Module to use
    // @end
    void SetEthernetModule(
      CnlLinkLayerInterface* const pEthernetModule );


    // @descr   Transmit a command to the link
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Command transmitted
    // @param   frame  IN: Frame to transmit
    // @end
    void TransmitFrame(
      const CnlFrame& frame );

    // @descr   Receive a command from the link
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Command received
    // @param   frame  IN: Frame to receive
    // @end
    void ReceiveFrame(
      const CnlFrame& frame );

    // @descr   Get the burn-in address
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Address returned
    // @return  Address
    // @end
    CString GetMacAddress() const;

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlNic();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlNic( const CnlNic& );
    CnlNic& operator=( const CnlNic& );
    CnlNic* operator&();
    const CnlNic* operator&() const;
};

#endif

