/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlOs.h
//
// @short    See nlOs.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>

//1 Local
#include "nlOs.h"
#include "nlSystemMgrType.h"
#include "nlUdpModule.h"
#include "nlTcpModule.h"
#include "nlIpModuleEndMachine.h"
#include "nlIpModuleRouter.h"
#include "nlEthernetModule.h"
#include "nlCompositeHandle.h"
#include "nlFrame.h"
#include "nlNetworkModel.h"
#include "nlSystemMgr.h"
#include "nlHwSwInterface.h"
#include "nlIPv4Address.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlOs::SnlImp* CnlOs::Imp()       { return m_pImp; }
inline const  CnlOs::SnlImp* CnlOs::Imp() const { return m_pImp; }

struct CnlOs::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlOs* const pOwner,
    const CnlCompositeHandle hMachine,
    CnlHwSwInterface* const pHardware );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlOs* const m_pOwner;

  // User handle for the hardware that owns this Os
  const CnlCompositeHandle m_hMachine;

  // Connection to the hardware
  CnlHwSwInterface* const m_pHardware;
  
  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The UDP Module (end machines only)
  CnlTcpModule* m_pTcpModule;

  // The TCP Module (end machines only)
  CnlUdpModule* m_pUdpModule;
  
  // The End Machine IP Module (end machines only)
  CnlIpModuleEndMachine* m_pIpModuleEndMachine;

  // The Router IP Module (routers only)
  CnlIpModuleRouter* m_pIpModuleRouter;

  // The generic interface I use to talk to my IP module
  CnlIpModule* m_pIpModuleInterface;
    
  // The Ethernet Module
  CnlEthernetModule* m_pEthernetModule;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlOs::SnlImp::SnlImp(
  CnlOs* const pOwner,
  const CnlCompositeHandle hMachine,
  CnlHwSwInterface* const pHardware )
 :
  m_pOwner( pOwner ),
  m_hMachine( hMachine ),
  m_pHardware( pHardware ),
  m_pTcpModule( NULL ),
  m_pUdpModule( NULL ),
  m_pIpModuleEndMachine( NULL ),
  m_pIpModuleRouter( NULL ),
  m_pIpModuleInterface( NULL ),
  m_pEthernetModule( NULL )
{
  switch( hMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      m_pTcpModule = new CnlTcpModule( m_pOwner );
      m_pUdpModule = new CnlUdpModule( m_pOwner );
      m_pIpModuleEndMachine = new CnlIpModuleEndMachine( m_pOwner);
      m_pIpModuleInterface = m_pIpModuleEndMachine;
      break;
    }
    case HANDLE_TYPE_ROUTER :
    {
      m_pIpModuleRouter = new CnlIpModuleRouter( m_pOwner);
      m_pIpModuleInterface = m_pIpModuleRouter;
      break;
    }
    default : assert( 0 );  // Don't know this type
  }

  //2 These don't vary
//  m_pArpModule = new CnlArpModule( m_pOwner );
  m_pEthernetModule = new CnlEthernetModule( m_pOwner );

}

///////////////////////////////////////////////////////////////////////////////
CnlOs::SnlImp::~SnlImp()
{
  delete m_pEthernetModule;
//  delete m_pArpModule;
  delete m_pIpModuleEndMachine;
  delete m_pIpModuleRouter;
  delete m_pUdpModule;
  delete m_pTcpModule;
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlOs::CnlOs(
  const CnlCompositeHandle hMachine,
  CnlHwSwInterface* const pHardware )
{
  m_pImp = new SnlImp( this, hMachine, pHardware );
}

///////////////////////////////////////////////////////////////////////////////
CnlOs::~CnlOs()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::Initialize()
{
  //1 Initialize my subcomponents
  switch( Imp()->m_hMachine.Type() )
  {
    case HANDLE_TYPE_END_MACHINE :
    {
      Imp()->m_pUdpModule->Initialize( Imp()->m_pIpModuleEndMachine );
      Imp()->m_pTcpModule->Initialize( Imp()->m_pIpModuleEndMachine );
      Imp()->m_pIpModuleEndMachine->Initialize(
        Imp()->m_pTcpModule, 
        Imp()->m_pUdpModule, 
//        Imp()->m_pArpModule,
        Imp()->m_pEthernetModule );
      break;
    }
    case HANDLE_TYPE_ROUTER :
    {
      Imp()->m_pIpModuleRouter->Initialize(
//        Imp()->m_pArpModule,
        Imp()->m_pEthernetModule );
      break;
    }
    default : assert( 0 );  // Don't know this type
  }

  //2 These don't vary
//  Imp()->m_pArpModule->Initialize(
//    Imp()->m_pIpModuleInterface,
//    Imp()->m_pEthernetModule );
  Imp()->m_pEthernetModule->Initialize( 
    Imp()->m_pIpModuleInterface );
//    Imp()->m_pArpModule );
  
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
CnlHwSwInterface* CnlOs::GetHardwareInterface() const
{
  return Imp()->m_pHardware;
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::SetConsoleMessage(
  const CString strMessage )
{
  CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  pNetworkModel->SetConsoleMessage( Imp()->m_hMachine, strMessage );
}

 ///////////////////////////////////////////////////////////////////////////////
EnlResult CnlOs::AssignIpAddress(
  const unsigned int nNicId,
  const CString& strClasslessIpAddress )
{
  const unsigned int nNumNics = Imp()->m_pHardware->GetNumNics();
  
  if (  ( nNicId < 0 )
     || ( nNicId >= nNumNics ) )
  {
    return RESULT_ERROR_NIC_DOES_NOT_EXIST;
  }

  return Imp()->m_pIpModuleInterface->AssignIpAddress( nNicId, strClasslessIpAddress );
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlOs::GetIpAddress(
  CString& strIpAddress,
  const unsigned int nNicOffset ) const
{
  CnlIPv4Address address;
  const EnlResult eResult = 
    Imp()->m_pIpModuleInterface->GetIpAddress( address, nNicOffset );
  strIpAddress = address.Machine();
  return eResult;
}


 ///////////////////////////////////////////////////////////////////////////////
CString CnlOs::GetArpCacheString() const
{
  return Imp()->m_pIpModuleInterface->GetCacheString();
}

 ///////////////////////////////////////////////////////////////////////////////
CString CnlOs::GetNetstatString() const
{
  return Imp()->m_pIpModuleInterface->GetNetstatString();
}

///////////////////////////////////////////////////////////////////////////////
EnlResult CnlOs::ReceiveFrame(
  const CnlFrame& frame )
{
  //1 Error checking
  if ( frame.GetFrameType() != LINK_PROTOCOL_ETHERNET )
  {
    return RESULT_ERROR_UNKNOWN_FRAME_TYPE;
  }

  Imp()->m_pEthernetModule->ReceiveFrame( frame );
  return RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::ReceiveEndPayload(
  const CnlVariableParmContainer& endPayload )
{
#if 0
  switch( endPayload.GetType() )
  {
    case COMMAND_ARP :
    {
      CString strMessage;
      strMessage.Format( 
        _T( "End Machine %d Os received command from Transport Module:" )
        _T( "\nArp: Who has %s? Tell %s" ),
        Imp()->m_hMachine.ID(),
        endPayload.GetParameter( 0 ),
        endPayload.GetParameter( 1 ) );
      
      CnlNetworkModel* const pNetworkModel = 
        CnlSystemMgr::Instance().GetInternalInterface();
      pNetworkModel->SetPofMessage( strMessage );
      pNetworkModel->SetPofHandle( Imp()->m_hMachine );

      if ( Imp()->IsMyIpAddress( endPayload.GetParameter( 0 ) ) )
      {
        Imp()->ArpAck( endPayload.GetParameter( 0 ), endPayload.GetParameter( 1 ) );
      }
      break;
    }

    default : 
    {
      assert( 0 );  // Don't know this command
    }
  }
#endif  // #if 0
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::SendArp(
  const CString& strDestinationIp )
{
  //1 Tell the Link layer to generate this command
  Imp()->m_pIpModuleInterface->SendArp( strDestinationIp );
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::SendPing(
  const CString& strDestinationIp )
{
  //1 Tell the Network layer to generate this command
  Imp()->m_pIpModuleInterface->SendPing( strDestinationIp );
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::ClearArpCache()
{
  Imp()->m_pIpModuleInterface->ClearArpCache();
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::Boot()
{
  ClearArpCache();
  Imp()->m_pIpModuleInterface->BuildRoutingTable();
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::VerifyArpResult(
  const CString& strDestinationIp )
{
  Imp()->m_pIpModuleInterface->VerifyArpResult( strDestinationIp );
}

///////////////////////////////////////////////////////////////////////////////
void CnlOs::VerifyPingResult(
  const CString& strDestinationIp )
{
  Imp()->m_pIpModuleInterface->VerifyPingResult( strDestinationIp );
}



