/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlPacket.h
//
// @short    See nlPacket.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>

//1 Local
#include "nlPacket.h"
#include "nlSegment.h"
#include "nlVariableParmContainer.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlPacket::SnlImp* CnlPacket::Imp()       { return m_pImp; }
inline const  CnlPacket::SnlImp* CnlPacket::Imp() const { return m_pImp; }

struct CnlPacket::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlPacket* const pOwner,
    const CString& strSourceIpAddress,
    const CString& strDestinationIpAddress,
    const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
    const CnlSegment pTcpSegment );

  // Copy constructor
  SnlImp( 
    CnlPacket* const pOwner,
    const SnlImp& rhs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlPacket* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // My source IP address
  const CString m_strSourceIpAddress;

  // My destination IP address
  const CString m_strDestinationIpAddress;
  
  // My payload's type
  const NnlSystemMgrType::EnlTransportProtocol m_eSegmentType;

  // The upper-layer segment I contain, if any
  const CnlSegment m_Segment;

  // Validity check
  const bool m_bIsValid;
  
private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlPacket::SnlImp::SnlImp(
  CnlPacket* const pOwner,
  const CString& strSourceIpAddress,
  const CString& strDestinationIpAddress,
  const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
  const CnlSegment segment )
 :
  m_pOwner( pOwner ),
  m_strSourceIpAddress( strSourceIpAddress ),
  m_strDestinationIpAddress( strDestinationIpAddress ),
  m_eSegmentType( eSegmentType ),
  m_Segment( segment ),
  m_bIsValid(  ( strSourceIpAddress != STR_NULL )
            && ( strDestinationIpAddress != STR_NULL )
            && ( eSegmentType != TRANSPORT_PROTOCOL_UNKNOWN ) )

{
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket::SnlImp::SnlImp( 
  CnlPacket* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( pOwner ),
  m_strSourceIpAddress( rhs.m_strSourceIpAddress ),
  m_strDestinationIpAddress( rhs.m_strDestinationIpAddress ),
  m_eSegmentType( rhs.m_eSegmentType ),
  m_Segment( rhs.m_Segment ),
  m_bIsValid( rhs.m_bIsValid )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlPacket::CnlPacket()
{
  m_pImp = new SnlImp( 
    this, 
    STR_NULL,
    STR_NULL,
    TRANSPORT_PROTOCOL_UNKNOWN, 
    CnlSegment() );
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket::CnlPacket(
  const CString& strSourceIpAddress,
  const CString& strDestinationIpAddress,
  const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
  const CnlSegment& segment )
{
  m_pImp = new SnlImp( 
    this, 
    strSourceIpAddress,
    strDestinationIpAddress,
    eSegmentType, 
    segment );
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket::~CnlPacket()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket::CnlPacket(
  const CnlPacket& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlPacket& CnlPacket::operator=( 
  const CnlPacket& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlPacket::GetSourceIpAddress() const
{
  return Imp()->m_strSourceIpAddress;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlPacket::GetDestinationIpAddress() const
{
  return Imp()->m_strDestinationIpAddress;
}

///////////////////////////////////////////////////////////////////////////////
EnlTransportProtocol CnlPacket::GetTransportProtocol() const
{
  return Imp()->m_eSegmentType;
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment CnlPacket::GetPayload() const
{
  return Imp()->m_Segment;
}

///////////////////////////////////////////////////////////////////////////////
bool CnlPacket::IsValid() const
{
  return Imp()->m_bIsValid;
}


