//////////////////////////////////////////////////////////////////////////////
//
// @descr    Network-layer Packet
//
// @short    Network-layer Packet
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_PACKET_H__
#define __NL_PACKET_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlSegment;
class CnlVariableParmContainer;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlPacket
{
  public:
    //1 Final
    // @descr  Default Constructor
    // @end
    explicit CnlPacket();

    // @descr  Constructor
    // @param  strSourceIpAddress  IN: My IP Address
    // @param  strDestinationIpAddress  IN: IP address of the destination machine
    // @param  eSegmentType  IN: The type of segment in my payload
    // @param  segment  IN: My payload
    // @end
    explicit CnlPacket(
      const CString& strSourceIpAddress,
      const CString& strDestinationIpAddress,
      const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
      const CnlSegment& segment );

    // @descr  Constructor
    // @param  strSourceIpAddress  IN: My IP Address
    // @param  strDestinationIpAddress  IN: IP address of the destination machine
    // @param  eSegmentType  IN: The type of segment in my payload
    // @param  vpc  IN: My payload
    // @end
//    explicit CnlPacket(
//      const CString& strSourceIpAddress,
//      const CString& strDestinationIpAddress,
//      const NnlSystemMgrType::EnlTransportProtocol eSegmentType,
//      const CnlVariableParmContainer& vpc );

    // @descr  Copy constructor
    // @end
    CnlPacket( 
      const CnlPacket& rhs );

    // @descr  Assignment operator
    // @end
    CnlPacket& operator=(
      const CnlPacket& rhs );

    // @descr   Get the source IP address from the header
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Address returned
    // @return  Source IP address
    // @end
    CString GetSourceIpAddress() const;

    // @descr   Get the destiination IP address from the header
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Address returned
    // @return  Destiination IP address
    // @end
    CString GetDestinationIpAddress() const;

    // @descr   Get the segment type. 
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Type returned.
    // @return  Type
    // @end
    NnlSystemMgrType::EnlTransportProtocol GetTransportProtocol() const;

    // @descr   Extract the segment payload
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Segment extracted
    // @return  Segment
    // @end
    CnlSegment GetPayload() const;

    // @descr   Extract the segment payload
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Segment extracted
    // @return  Segment
    // @end
//    CnlVariableParmContainer GetVpcPayload() const;


    bool IsValid() const;
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlPacket();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlPacket* operator&();
    const CnlPacket* operator&() const;
};

#endif

