//////////////////////////////////////////////////////////////////////////////
//
// @descr    Interface to User Processes
//
// @short    Interface to User Processes
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_PROCESS_INTERFACE_H__
#define __NL_PROCESS_INTERFACE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlVariableParmContainer;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlProcessInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @param   nX  IN: Desc
    //            (cont'd)
    // @end
    explicit CnlProcessInterface()
    {}

    // @descr  Post-constructor initialization
    // @end
    void Initialize();
//
    // @descr  Pre-Destructor cleanup
    // @end
//    void PreDestroy();

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlProcessInterface()
{}

    //2 From Cnl

    //1 Abstract
    // @descr   Receive a user payload from the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload processed
    // @param   payload  IN: Payload to process
    // @end
    virtual void ReceiveEndPayload(
      const CnlVariableParmContainer& payload ) = 0;

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlProcessInterface( const CnlProcessInterface& );
    CnlProcessInterface& operator=( const CnlProcessInterface& );
    CnlProcessInterface* operator&();
    const CnlProcessInterface* operator&() const;
};

#endif

