//////////////////////////////////////////////////////////////////////////////
//
// @descr    A host in a subnet
//
// @short    A host in a subnet
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_END_MACHINE_H__
#define __NL_END_MACHINE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local
#include "nlNetworkLeaf.h"
#include "nlHwSwInterface.h"
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlNic;
class CnlCompositeHandle;
class CnlCommand;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlProcessingMachine : public CnlNetworkLeaf, CnlHwSwInterface
{
  public:
    //1 Final
    // @descr  End Machine Constructor
    //         Automatically creates and installs 1 Nic
    // @param  hThis  IN: Handle to this object
    // @param  eType  IN: Type of machine to create
    // @end
    explicit CnlProcessingMachine(
      CnlCompositeHandle hThis,
      const NnlSystemMgrType::EnlMachineType eType );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();
    
    // @descr   Get a pointer to the specified Nic.
    //          Preconditions:
    //            1. NIC exists
    //          Postconditions(dependencies):
    //            a. (all) Pointer returned; else NULL returned.
    // @param   nNicId  IN: Which NIC to get
    // @return  Pointer or NULL
    // @end
    CnlNic* GetNic(
      const unsigned int nNicId ) const;

    //2 Forwarded by CnlNetworkModel
    NnlSystemMgrType::EnlResult AssignIpAddress(
      const unsigned int,
      const CString& );
    NnlSystemMgrType::EnlResult ExecuteCommand(
      const CnlCommand* const );
    NnlSystemMgrType::EnlResult PostExecuteCommand(
      const CnlCommand* const );
    NnlSystemMgrType::EnlResult GetIpAddress(
      CString&,
      const unsigned int ) const;
    NnlSystemMgrType::EnlResult GetMacAddress(
      CString&,
      const unsigned int ) const;
    NnlSystemMgrType::EnlResult GetNicCount(
      unsigned int& ) const;
    NnlSystemMgrType::EnlResult GetString(
      CString&,
      NnlSystemMgrType::EnlCommandLineEntry ) const;
    NnlSystemMgrType::EnlResult TogglePower();
    NnlSystemMgrType::EnlResult GetPowerStatus(
      bool& ) const;
    NnlSystemMgrType::EnlResult ClearArpCache();

    //2 From CnlHardwareInterface
    bool IsMachineOn() const;
    CnlNic* GetNic(
      const unsigned int nNicId );
    CnlOs* GetOs() const;
    CnlCompositeHandle GetProcessingMachineHandle() const;
    unsigned int GetNumNics() const;
    NnlSystemMgrType::EnlResult AddNic(
      NnlSystemMgrType::EnlRouterNicType eNicType );

    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlProcessingMachine();

    
    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlProcessingMachine( const CnlProcessingMachine& );
    CnlProcessingMachine& operator=( const CnlProcessingMachine& );
    CnlProcessingMachine* operator&();
    const CnlProcessingMachine* operator&() const;
};

#endif

