//////////////////////////////////////////////////////////////////////////////
//
// @descr    The routing table used by the Os to route IP packets
//
// @short    SThe routing table used by the Os to route IP packets
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_ROUTING_TABLE_H__
#define __NL_ROUTING_TABLE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlSystemMgrType.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlRoutingTableRow;
class CnlIPv4Address;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlRoutingTable
{
  public:
    //1 Final
    // @descr  Constructor
    // @param   nX  IN: Desc
    //            (cont'd)
    // @end
    explicit CnlRoutingTable();

    // @descr  Post-constructor initialization
    // @end
    void AddRow(
      const CString strDestination,
      const CString strMask,
      const CString strGateway,
      const CString strInterface );

    // @descr   Return the contents of the table in displayable form
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Description returned
    // @return  Description
    // @end
    CString AsString() const;

    // @descr   Lookup a destination IP address
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Output variables set to best matching-line
    // @param   strGateway  OUT: "Gateway" column value
    // @param   strInterface  OUT: "Interface" column value
    // @param   strIpToLookup  IN: Address to find
    // @return  True if a match was found; False otherwise
    // @end
    bool Lookup( 
      CString& strGateway,
      CString& strInterface,
      const CString& strIpToLookup ) const;

    void CnlRoutingTable::ClearTable();

    //2 Forwarded by CnlIpModule
    NnlSystemMgrType::EnlResult CnlRoutingTable::AssignIpAddress(
      const unsigned int nNicId,
      const CString& address );
    unsigned int GetNicOffsetFromInterface( 
      const CString& ) const;
    NnlSystemMgrType::EnlResult GetIpAddress(
      CnlIPv4Address& address,
      const unsigned int nNicOffset ) const;
    bool IsMyIpAddress(
      const CString& strAddress ) const;
    CString GetBroadcastAddressFromInterface(
      const CString& ) const;

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlRoutingTable();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlRoutingTable( const CnlRoutingTable& );
    CnlRoutingTable& operator=( const CnlRoutingTable& );
    CnlRoutingTable* operator&();
    const CnlRoutingTable* operator&() const;
};

#endif

