/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlRoutingTableRow.h
//
// @short    See nlRoutingTableRow.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From Utilities
#include <nlCommonType.h>

//1 Local
#include "nlRoutingTableRow.h"
#include "nlIPv4Address.h"
#include "nlSystemMgrType.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlType;
using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlRoutingTableRow::SnlImp* CnlRoutingTableRow::Imp()       { return m_pImp; }
inline const  CnlRoutingTableRow::SnlImp* CnlRoutingTableRow::Imp() const { return m_pImp; }

struct CnlRoutingTableRow::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlRoutingTableRow* const pOwner,
      const CString& destination,
      const CString& netMask,
      const CString& gateway,
      const CString& interface );

  SnlImp(
    CnlRoutingTableRow* const pOwner,
    const SnlImp& rhs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlRoutingTableRow* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The Network Destination column
  const CString m_strDestination;

  // The NetMask column
  const CString m_strNetMask;

  // The Gateway column
  const CString m_strGateway;

  // The Interface column
  const CString m_strInterface;

private:

  //1 Unimplemented
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::SnlImp::SnlImp(
  CnlRoutingTableRow* const pOwner,
    const CString& destination,
    const CString& netMask,
    const CString& gateway,
    const CString& interface )
 :
  m_pOwner( pOwner ),
  m_strDestination( destination ),
  m_strNetMask( netMask ),
  m_strGateway( gateway ),
  m_strInterface( interface )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::SnlImp::SnlImp(
  CnlRoutingTableRow* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( rhs.m_pOwner ),
  m_strDestination( rhs.m_strDestination ),
  m_strNetMask( rhs.m_strNetMask ),
  m_strGateway( rhs.m_strGateway ),
  m_strInterface( rhs.m_strInterface )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::CnlRoutingTableRow()
{
  m_pImp = new SnlImp( 
    this, 
    STR_NULL, 
    STR_NULL, 
    STR_NULL, 
    STR_NULL );
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::CnlRoutingTableRow(
  const CString& destination,
  const CString& netMask,
  const CString& gateway,
  const CString& interface )
{
  m_pImp = new SnlImp(  this, destination, netMask, gateway, interface );
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::CnlRoutingTableRow(
  const CnlRoutingTableRow& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow& CnlRoutingTableRow::operator=(
  const CnlRoutingTableRow& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;  
}

///////////////////////////////////////////////////////////////////////////////
CnlRoutingTableRow::~CnlRoutingTableRow()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlRoutingTableRow::AsString() const
{
  CString strDescription;
  strDescription.Format(
    strNetstatFormat,
    Imp()->m_strDestination,
    Imp()->m_strNetMask,
    Imp()->m_strGateway,
    Imp()->m_strInterface );

  return strDescription;
}

///////////////////////////////////////////////////////////////////////////////
unsigned long CnlRoutingTableRow::GetDestination() const
{
  const CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  unsigned long nDestination = 0;
  const EnlResult eResult = pNetworkModel->DotToNumeric( 
    nDestination, 
    Imp()->m_strDestination, 
    IP_VERSION_4 );

  return nDestination;
}

///////////////////////////////////////////////////////////////////////////////
unsigned long CnlRoutingTableRow::GetNetmask() const
{
  const CnlNetworkModel* const pNetworkModel = 
    CnlSystemMgr::Instance().GetInternalInterface();
  unsigned long nNetMask = 0;
  const EnlResult eResult = pNetworkModel->DotToNumeric( 
    nNetMask, 
    Imp()->m_strNetMask, 
    IP_VERSION_4 );

  return nNetMask;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlRoutingTableRow::GetGateway() const
{
  return Imp()->m_strGateway;
}

///////////////////////////////////////////////////////////////////////////////
CString CnlRoutingTableRow::GetInterface() const
{
  return Imp()->m_strInterface;
}


