//////////////////////////////////////////////////////////////////////////////
//
// @descr    One row of the routing table
//
// @short    One row of the routing table
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_ROUTING_TABLE_ROW_H__
#define __NL_ROUTING_TABLE_ROW_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlIPv4Address;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlRoutingTableRow
{
  public:
    //1 Final
    // @descr  Default constructor
    // @end
    explicit CnlRoutingTableRow();

    // @descr  Constructor
    // @param   nX  IN: Desc
    //            (cont'd)
    // @end
    explicit CnlRoutingTableRow(
      const CString& destination,
      const CString& netMask,
      const CString& gateway,
      const CString& interface );

    // Copy constructor
    CnlRoutingTableRow(
      const CnlRoutingTableRow& rhs );

    // Assignment operator
    CnlRoutingTableRow& operator=(
      const CnlRoutingTableRow& rhs );

    // @descr  Return the row as a string
    // @end
    CString AsString() const;

    // @descr  Get the destination column value
    // @end
    unsigned long GetDestination() const;

    // @descr  Get the netmask column value
    // @end
    unsigned long GetNetmask() const;

    // @descr  Get the gateway column value
    // @end
    CString GetGateway() const;

    // @descr  Get the interface column value
    // @end
    CString GetInterface() const;

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlRoutingTableRow();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlRoutingTableRow* operator&();
    const CnlRoutingTableRow* operator&() const;
};

#endif

