/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlSegment.h
//
// @short    See nlSegment.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlSegment.h"
#include "nlSystemMgrType.h"
#include "nlVariableParmContainer.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlSegment::SnlImp* CnlSegment::Imp()       { return m_pImp; }
inline const  CnlSegment::SnlImp* CnlSegment::Imp() const { return m_pImp; }

struct CnlSegment::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlSegment* const pOwner,
    const int nSourcePort,
    const CnlVariableParmContainer& payload);

  // Copy constructor
  SnlImp( 
    CnlSegment* const pOwner,
    const SnlImp& rhs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlSegment* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The port of the source process
  const int m_nSourcePort;

  // My end payload
  const CnlVariableParmContainer m_payload;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlSegment::SnlImp::SnlImp(
  CnlSegment* const pOwner,
  const int nSourcePort,
  const CnlVariableParmContainer& payload )
 :
  m_pOwner( pOwner ),
  m_nSourcePort( nSourcePort ),
  m_payload( payload )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment::SnlImp::SnlImp( 
  CnlSegment* const pOwner,
  const SnlImp& rhs )
 :
  m_pOwner( rhs.m_pOwner ),
  m_nSourcePort( rhs.m_nSourcePort ),
  m_payload( rhs.m_payload )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlSegment::CnlSegment()
{
  m_pImp = new SnlImp( this, 0, CnlVariableParmContainer() );
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment::CnlSegment(
  const int nSourcePort,
  const CnlVariableParmContainer& payload )
{
  m_pImp = new SnlImp( this, nSourcePort, payload );
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment::~CnlSegment()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlSegment::Initialize()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlSegment::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment::CnlSegment(
  const CnlSegment& rhs )
{
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
}

///////////////////////////////////////////////////////////////////////////////
CnlSegment& CnlSegment::operator=( 
  const CnlSegment& rhs )
{
  delete m_pImp;
  m_pImp = new SnlImp( this, *( rhs.m_pImp ) );
  return *this;
}

///////////////////////////////////////////////////////////////////////////////
CnlVariableParmContainer CnlSegment::GetPayload() const
{
  return Imp()->m_payload;
}



