//////////////////////////////////////////////////////////////////////////////
//
// @descr    Transport-layer segment
//
// @short    Transport-layer segment
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_SEGMENT_H__
#define __NL_SEGMENT_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlVariableParmContainer;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlSegment
{
  public:
    //1 Final
    // @descr  Default Constructor
    // @end
    explicit CnlSegment();

    // @descr  Constructor
    // @param  nSourcePort  IN: Port of process making this request
    // @param  payload  IN: Source process' payload
    // @end
    explicit CnlSegment(
      const int nSourcePort,
      const CnlVariableParmContainer& payload );

    // @descr  Copy constructor
    // @end
    CnlSegment( 
      const CnlSegment& rhs );

    // @descr  Assignmen operator
    // @end
    CnlSegment& operator=(
      const CnlSegment& rhs );

    // @descr  Post-constructor initialization
    // @end
    void Initialize();

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    // @descr   Extract the user payload
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload extracted
    // @return  Segment
    // @end
    CnlVariableParmContainer GetPayload() const;

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlSegment();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlSegment* operator&();
    const CnlSegment* operator&() const;
};

#endif

