/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlSystemMgr.h
//
// @short    See nlSystemMgr.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlSystemMgr.h"
#include "nlNetworkModel.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//using namespace xx;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlSystemMgr::SnlImp* CnlSystemMgr::Imp()       { return m_pImp; }
inline const  CnlSystemMgr::SnlImp* CnlSystemMgr::Imp() const { return m_pImp; }

struct CnlSystemMgr::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlSystemMgr* const pOwner );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // Handle to the object that owns this implementation
  CnlSystemMgr* const m_pOwner;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // Singleton-related members
  static CnlSystemMgr* s_pInstance;
  static bool s_bBeenCreated;

  // The object that implements the interfaces
  CnlNetworkModel* const m_pNetworkModel;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlSystemMgr::SnlImp::SnlImp(
  CnlSystemMgr* const pOwner )
 :
  m_pOwner( pOwner ),
  m_pNetworkModel( new CnlNetworkModel() )
{
//  assert( m_pNetworkModel->UnitTest() );
}

///////////////////////////////////////////////////////////////////////////////
CnlSystemMgr::SnlImp::~SnlImp()
{
  delete m_pNetworkModel;
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////
CnlSystemMgr* CnlSystemMgr::SnlImp::s_pInstance = NULL;
bool CnlSystemMgr::SnlImp::s_bBeenCreated = false;

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlSystemMgr::CnlSystemMgr()
{
  m_pImp = new SnlImp( this );
}

///////////////////////////////////////////////////////////////////////////////
CnlSystemMgr::~CnlSystemMgr()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlSystemMgr::Create()
{
  //2  The first time through, the singleton instance pointer is NULL.
  if ( CnlSystemMgr::SnlImp::s_bBeenCreated == false)
  {
    CnlSystemMgr::SnlImp::s_pInstance = new CnlSystemMgr;
    CnlSystemMgr::SnlImp::s_bBeenCreated = true;
  }
  else
  {
    //3  No one should ever try to re-create a singleton
    //3  If this assert is asserted, find the culprit elsewhere!
    assert(0);
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlSystemMgr::Destroy()
{
  if (  CnlSystemMgr::SnlImp::s_bBeenCreated 
    &&  CnlSystemMgr::SnlImp::s_pInstance != NULL )
  {
    delete CnlSystemMgr::SnlImp::s_pInstance;
    CnlSystemMgr::SnlImp::s_pInstance = NULL;
  }
  else
  {
    //3  This implies deletion before creation or re-creation;not allowed
    //3  If this assert is asserted, find the culprit elsewhere!
    assert( 0 );
  }
}

///////////////////////////////////////////////////////////////////////////////
CnlSystemMgr& CnlSystemMgr::Instance()
{
  if (CnlSystemMgr::SnlImp::s_pInstance == NULL)
  {
    //3 We manually manage singleton lifetimes in this project
    assert(0);
  }
  
  return *CnlSystemMgr::SnlImp::s_pInstance;
}

///////////////////////////////////////////////////////////////////////////////
CnlClientInterface* CnlSystemMgr::GetClientInterface() const
{
  return Imp()->m_pNetworkModel;
}

///////////////////////////////////////////////////////////////////////////////
CnlNetworkModel* CnlSystemMgr::GetInternalInterface() const
{
  return Imp()->m_pNetworkModel;
}




