//////////////////////////////////////////////////////////////////////////////
//
// @descr    Singleton: Handles creation/destruction/access of high-level objects
//           This class serves as an interface, while CnlNetworkMOdel handles
//           the implementation.
//
// @short    Singleton for accessing the network model
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __nl_SYSTEM_MGR_H__
#define __nl_SYSTEM_MGR_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local
#include "nlClientInterface.h"
#include "nlNetworkModel.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlSystemMgr
{
  public:
    //1 Static
    
    // @descr   Creates the one and only instance of this class
    // @end
    static void Create();

    //  @descr   Destroys the one and only instance of this class
    //  @end
    static void Destroy();

    //  @descr   Gets a reference to the one and only instance of this class
    //  @end
    static CnlSystemMgr& Instance();

    //1 Final
    // @descr   Access functionality for clients (dialogs, RPI users)
    //          Preconditons: None
    //          Postconditions(dependencies):
    //            a. (none) Handle returned
    // @return  Handle
    // @end
    //
    CnlClientInterface* GetClientInterface() const;
    
    // @descr   Access functionality for internal use only (by classes
    //          existing inside the CnlSystemMgr project).
    //          Note: Do not publish CagNetworktModel.h to the SDK! (See file)
    //          Preconditons: None
    //          Postconditions(dependencies):
    //            a. (none) Handle returned
    // @return  Handle
    // @end
    //
    CnlNetworkModel* GetInternalInterface() const;

  protected:

  private:

    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlSystemMgr();

    // @descr  Destructor
    // @end
    ~CnlSystemMgr();

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlSystemMgr( const CnlSystemMgr& );
    CnlSystemMgr& operator=( const CnlSystemMgr& );
    CnlSystemMgr* operator&();
    const CnlSystemMgr* operator&() const;
};

#endif

