//////////////////////////////////////////////////////////////////////////////
//
// @descr    SystemMgr typedefs, enums & helper classes
//
// @short    SystemMgr typedefs, enums & helper classes
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_SYSTEMMGR_TYPE_H__
#define __NL_SYSTEMMGR_TYPE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

// From 

// Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
namespace NnlSystemMgrType
{
  //1 Constants
  // Assumption: IPv4
  const int nNumBitsInIPv4Address = 32;

  // Reserved for: subnet, broadcast, router
  const int nNumReservedAddressesPerSubnet = 3;

  // Ensures that after reserved subnet addresses are used,
  // there is room for at least 1 host
  const int nMinNumHostBits = 2;

  // Limits the size of a network
  const int nMaxNumHostBits = 8;

  // Limits number of end machines per network (ensures each can have 1 Nic)
  const int nMaxNumProcessingMachines = 
    1 << nMaxNumHostBits - nNumReservedAddressesPerSubnet;

  // Arbitrarily limits number of networks
  const int nMaxNumNetworks = 100;

  const CString strEthernetBroadcastAddress = _T( "ff:ff:ff:ff:ff:ff" );
  const CString strIpBroadcastAddress = _T( "255.255.255.255" );
  const CString strInterfaceLoopBack = _T( "127.0.0.1" );

  const CString strNetstatFormat = _T( "%-25s %-25s %-25s %-25s" );

  //1 Typedefs

  //1 Enums  
  enum EnlMessage
  {
    MESSAGE_UNKNOWN,
    MESSAGE_MODEL_CHANGED,
    MESSAGE_IP_ADDRESS_CHANGED,
    MESSAGE_SETTINGS_CHANGED,
    MESSAGE_POF_CHANGED,
    MESSAGE_POWER_STATUS_CHANGED,
    MESSAGE_COMMAND_STARTED,
    MESSAGE_COMMAND_COMPLETED,
  };

  enum EnlResult
  {
    RESULT_UNKNOWN,
      
    //2 Information level
    RESULT_OK,
    RESULT_EXECUTION_INCOMPLETE,
    RESULT_POF_PROCESSING_DISABLED,
    RESULT_LAYER_FILTERED_OUT,
    RESULT_EVENT_TYPE_FILTERED_OUT,
    
    //2 Warning level
    RESULT_WARNING_LEVEL,

    //2 Error level
    RESULT_ERROR_LEVEL,
    RESULT_ERROR_INVALID_ADDRESS,
    RESULT_ERROR_INVALID_ADDRESS_LENGTH,
    RESULT_ERROR_INVALID_HANDLE,
    RESULT_ERROR_WRONG_HANDLE_TYPE,
    RESULT_ERROR_NETWORK_DOES_NOT_EXIST,
    RESULT_ERROR_END_MACHINE_DOES_NOT_EXIST,
    RESULT_ERROR_NIC_DOES_NOT_EXIST,
    RESULT_ERROR_WRONG_NETWORK,
    RESULT_ERROR_DUPLICATE_ADDRESS,
    RESULT_ERROR_MAX_EXCEEDED,
    RESULT_ERROR_ALREADY_CONNECTED,
    RESULT_ERROR_NO_COMMAND_EXECUTING,
    RESULT_ERROR_PREVIOUS_EXECUTION_IMCOMPLETE,
    RESULT_ERROR_UNKNOWN_FRAME_TYPE    ,
    RESULT_ERROR_MACHINE_IS_POWERED_OFF,
    RESULT_ERROR_WRONG_MACHINE_TYPE,
    RESULT_ERROR_UNSUPPORTED_OPERATION
  };
  
  enum EnlCommand
  {
    COMMAND_UNKNOWN,
    COMMAND_ARP,
    COMMAND_PING
  };

  enum EnlTransportProtocol
  {
    TRANSPORT_PROTOCOL_UNKNOWN,
    TRANSPORT_PROTOCOL_NONE,
    TRANSPORT_PROTOCOL_ROUTER_QUERY,
    TRANSPORT_PROTOCOL_ROUTER_RESPONSE,
    TRANSPORT_PROTOCOL_PING,
    TRANSPORT_PROTOCOL_RPING,
    TRANSPORT_PROTOCOL_TCP,
    TRANSPORT_PROTOCOL_UDP
  };

  enum EnlNetworkProtocol
  {
    NETWORK_PROTOCOL_UNKNOWN,
    NETWORK_PROTOCOL_ARP,
    NETWORK_PROTOCOL_IP,
    NETWORK_PROTOCOL_RARP
  };

  enum EnlLinkProtocol
  {
    LINK_PROTOCOL_UNKNOWN,
    LINK_PROTOCOL_ETHERNET
  };

  enum EnlDirection
  {
    DIRECTION_UNKNOWN,
    DIRECTION_TO,
    DIRECTION_FROM
  };

  enum EnlAddressType
  {
    ADDRESS_TYPE_UNKNOWN,
    ADDRESS_TYPE_NETWORK,
    ADDRESS_TYPE_MACHINE
  };

  enum EnlIpVersion
  {
    IP_VERSION_UNKNOWN,
    IP_VERSION_4,
    IP_VERSION_6
  };

  enum EnlLayer
  {
    LAYER_UNKNOWN,
    LAYER_PHYSICAL,
    LAYER_LINK,
    LAYER_NETWORK,
    LAYER_TRANSPORT,
    LAYER_UPPER,
    NUM_LAYERS_PLUS_1
  };

  enum EnlCommandLineEntry
  {
    COMMAND_LINE_ENTRY_UNKNWON,
    COMMAND_LINE_ENTRY_ARP_A,
    COMMAND_LINE_ENTRY_ARP_D,
    COMMAND_LINE_ENTRY_NETSTAT_RN
  };

  enum EnlMachineType
  {
    MACHINE_TYPE_END_UNKNOWN,
    MACHINE_TYPE_END_MACHINE,
    MACHINE_TYPE_ROUTER
  };

  enum EnlRouterNicType
  {
    ROUTER_NIC_TYPE_UNKNOWN,

    //3 NICs used to route between known networks
    ROUTER_NIC_TYPE_NETWORK,

    //3 NIC used to communicate with the gateway router
    ROUTER_NIC_TYPE_GATEWAY
  };

  enum EnlProgress
  {
    PROGRESS_NOT_UNKNOWN,
    PROGRESS_NOT_STARTED,
    PROGRESS_STARTED,
    PROGRESS_COMPLETED
  };

  enum EnlEventType
  {
    EVENT_TYPE_UNKNOWN,
    EVENT_TYPE_INFO,
    EVENT_TYPE_SEND,
    EVENT_TYPE_RECEIVE_ACCEPT,
    EVENT_TYPE_RECEIVE_REJECT,
    NUM_EVENT_TYPES
  };

  enum EnlThreadStatus
  {
    THREAD_STATUS_UNKNOWN,
    THREAD_STATUS_RUNNING,
    THREAD_STATUS_SUSPENDED,
    THREAD_STATUS_COMPLETED
  };
}

#endif


