//////////////////////////////////////////////////////////////////////////////
//
// @descr    The TCP software module
//
// @short    The TCP software module
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_TCP_MODULE_H__
#define __NL_TCP_MODULE_H__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local
#include "nlTransportLayerInterface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlIpModule;
class CnlProcessInterface;
class CnlOs;

//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlTcpModule : public CnlTransportLayerInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @param  pOs  IN: The operating system core
    // @end
    CnlTcpModule(
      CnlOs* const pOs );

    // @descr  Post-constructor initialization
    // @end
    void Initialize(
      CnlNetworkLayerInterface* const );

    // @descr  Pre-Destructor cleanup
    // @end
    void PreDestroy();

    //2 From CnlTransportLayerInterface
    void Broadcast(
      const int,
      const CnlVariableParmContainer& );
    void SendToIpAddress(
      const int,
      const CnlVariableParmContainer&,
      const CString );
    void ReceiveSegment(
      const CnlSegment& );

    //2 From Cnl
    
    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlTcpModule();

    //2 From Cnl

    //1 Abstract

    //2 From Cnl

  protected:

  private:

    //1 Implementation
    struct SnlImp;
    SnlImp* m_pImp;
    SnlImp* Imp();
    const SnlImp* Imp() const;
    friend SnlImp;

    //1 Unimplemented
    CnlTcpModule( const CnlTcpModule& );
    CnlTcpModule& operator=( const CnlTcpModule& );
    CnlTcpModule* operator&();
    const CnlTcpModule* operator&() const;
};

#endif

