//////////////////////////////////////////////////////////////////////////////
//
// @descr    Interface for Transport Layer modules
//
// @short    Interface for Transport Layer modules
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __NL_TRANSPORT_LAYER_INTERFACE_H__
#define __NL_TRANSPORT_LAYER_INTERFACE__

///////////////////////////////////////////////////////////////////////////////
//
// Compiler & Linker Directives
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////

//1 External
//2 From 

//1 Local

//////////////////////////////////////////////////////////////////////////////
//
// Forward Declarations
//
//////////////////////////////////////////////////////////////////////////////
class CnlNetworkLayerInterface;
class CnlVariableParmContainer;
class CnlSegment;
class CnlProcessInterface;


//////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
//////////////////////////////////////////////////////////////////////////////
class CnlTransportLayerInterface
{
  public:
    //1 Final
    // @descr  Constructor
    // @end
    explicit CnlTransportLayerInterface()
    {}

    //1 Virtual
    // @descr  Destructor
    // @end
    virtual ~CnlTransportLayerInterface()
    {}

    //1 Abstract
    // @descr   Pass the specified payload down for broadcast in the network
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload passed down in a segment
    // @param   nSourcePort  IN: Port of process making request
    // @param   payload  IN: Payload to deliver to Packet layer
    // @end
    virtual void Broadcast(
      const int nSourcePort,
      const CnlVariableParmContainer& payload ) = 0;

    // @descr   Send the specified payload to the specified IP Address
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Payload sent
    // @param   nSourcePort  IN: Port of process making request
    // @param   payload  IN: Paylod to send
    // @param   strIpAddress  IN: Address to send to
    // @end
    virtual void SendToIpAddress(
      const int nSourcePort,
      const CnlVariableParmContainer& payload,
      const CString strIpAddress ) = 0 ;

    // @descr   Receive a segment from the packet layer module.
    //          Preconditions:
    //            1. None
    //          Postconditions(dependencies):
    //            a. (none) Segment processed
    // @param   segment  IN: Segment to process.
    // @param   strSenderIp  IN: Sender's IP Address
    // @end
    virtual void ReceiveSegment(
      const CnlSegment& segment ) = 0;


  protected:

  private:

    //1 Implementation
    //3 None

    //1 Unimplemented
    CnlTransportLayerInterface( const CnlTransportLayerInterface& );
    CnlTransportLayerInterface& operator=( const CnlTransportLayerInterface& );
    CnlTransportLayerInterface* operator&();
    const CnlTransportLayerInterface* operator&() const;
};

#endif

