/////////////////////////////////////////////////////////////////////////////
//
// @descr    See nlUdpModule.h
//
// @short    See nlUdpModule.h
//
// @author   Jim Carole 
//
// @end
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Include Files
//
//////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

//1 External
//2 From 

//1 Local
#include "nlUdpModule.h"
#include "nlSystemMgrType.h"
#include "nlSegment.h"
#include "nlNetworkLayerInterface.h"
#include "nlProcessInterface.h"
#include "nlVariableParmContainer.h"
#include "nlOs.h"

///////////////////////////////////////////////////////////////////////////////
//
// Global Variables and Namespace Declarations
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace NnlSystemMgrType;

///////////////////////////////////////////////////////////////////////////////
//
// File Local
//
///////////////////////////////////////////////////////////////////////////////
namespace
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Private Implementation
//
///////////////////////////////////////////////////////////////////////////////
inline        CnlUdpModule::SnlImp* CnlUdpModule::Imp()       { return m_pImp; }
inline const  CnlUdpModule::SnlImp* CnlUdpModule::Imp() const { return m_pImp; }

struct CnlUdpModule::SnlImp
{
  //1 Function Prototypes
  
  // Constructor
  SnlImp(
    CnlUdpModule* const pOwner,
    CnlOs* const pOs );

  // Destructor
   ~SnlImp();

  //1 Handles (Never 'new' or 'delete')

  // The object that owns this implementation
  CnlUdpModule* const m_pOwner;

  // The operating system that owns this module
  CnlProcessInterface* const m_pOs;

  //1 Coordinated Data (Responsible for either 'new' or 'delete')

  // Desc

  //1 Owned Data (Responsible for both 'new' and 'delete')

  // The IP module I use for sending traffic down
  CnlNetworkLayerInterface* m_pIpModule;

private:

  //1 Unimplemented
  SnlImp( const SnlImp& );
  SnlImp& operator=( const SnlImp& );
  SnlImp* operator&();
  const SnlImp* operator&() const;
};

//1 Function Definitions

///////////////////////////////////////////////////////////////////////////////
CnlUdpModule::SnlImp::SnlImp(
  CnlUdpModule* const pOwner,
  CnlOs* const pOs )
 :
  m_pOwner( pOwner ),
  m_pOs( pOs ),
  m_pIpModule( NULL )
{
}

///////////////////////////////////////////////////////////////////////////////
CnlUdpModule::SnlImp::~SnlImp()
{
}

///////////////////////////////////////////////////////////////////////////////
//
// Static Member Initialization
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// Public/Protected Implementation
//
///////////////////////////////////////////////////////////////////////////////
CnlUdpModule::CnlUdpModule(
  CnlOs* const pOs )
 :
  CnlTransportLayerInterface()
{
  m_pImp = new SnlImp( this, pOs );
}

///////////////////////////////////////////////////////////////////////////////
CnlUdpModule::~CnlUdpModule()
{
  delete m_pImp;
}

///////////////////////////////////////////////////////////////////////////////
void CnlUdpModule::Initialize(
  CnlNetworkLayerInterface* const pPacketLayerModule )
{
  if ( Imp()->m_pIpModule != NULL )
  {
    Imp()->m_pIpModule = pPacketLayerModule;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CnlUdpModule::PreDestroy()
{
}

///////////////////////////////////////////////////////////////////////////////
void CnlUdpModule::Broadcast(
  const int nSourcePort,
  const CnlVariableParmContainer& payload )
{
  //1 Make a UDP segment containing this payload
  CnlSegment segment( nSourcePort, payload );

  //1 Send it down  
//  Imp()->m_pIpModule->Broadcast( TRANSPORT_PROTOCOL_UDP, segment );
}

///////////////////////////////////////////////////////////////////////////////
void CnlUdpModule::SendToIpAddress(
  const int nSourcePort,
  const CnlVariableParmContainer& payload,
  const CString strIpAddress )
{
  //1 Make a UDP segment containing this payload
  CnlSegment segment( nSourcePort, payload );

  //1 Send it down  
//  Imp()->m_pIpModule->SendToIpAddress( TRANSPORT_PROTOCOL_UDP, segment, strIpAddress );
}

///////////////////////////////////////////////////////////////////////////////
void CnlUdpModule::ReceiveSegment(
  const CnlSegment& segment )
{
  if ( 0 )  // Not one of my Ports
  {
    return;
  }

  Imp()->m_pOs->ReceiveEndPayload( segment.GetPayload() );
}

